# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

from BitTorrent.platform import bttime as time

import const

from khash import intify
from ktable import KTable, K
from util import unpackNodes
from krpc import KRPCProtocolError, KRPCSelfNodeError
from bisect import insort

class NodeWrap(object):
    def __init__(self, node, target):
        self.num = target
        self.node = node

    def __cmp__(self, o):
        """ this function is for sorting nodes relative to the ID we are looking for """
        x, y = self.num ^ o.num, self.num ^ self.node.num
        if x > y:
            return 1
        elif x < y:
            return -1
        return 0
        
class ActionBase(object):
    """ base class for some long running asynchronous proccesses like finding nodes or values """
    def __init__(self, table, target, callback, callLater):
        self.table = table
        self.target = target
        self.callLater = callLater
        self.num = intify(target)
        self.found = {}
        self.foundq = []
        self.queried = {}
        self.queriedip = {}
        self.answered = {}
        self.callback = callback
        self.outstanding = 0
        self.finished = 0
    
    def sort(self, a, b):
        """ this function is for sorting nodes relative to the ID we are looking for """
        x, y = self.num ^ a.num, self.num ^ b.num
        if x > y:
            return 1
        elif x < y:
            return -1
        return 0

    def shouldQuery(self, node):
        if node.id == self.table.node.id:
            return False
        elif (node.host, node.port) not in self.queriedip and node.id not in self.queried:
            self.queriedip[(node.host, node.port)] = 1
            self.queried[node.id] = 1
            return True
        return False
    
    def _cleanup(self):
        self.foundq = None
        self.found = None
        self.queried = None
        self.queriedip = None

    def goWithNodes(self, t):
        pass
    
    

FIND_NODE_TIMEOUT = 15

class FindNode(ActionBase):
    """ find node action merits it's own class as it is a long running stateful process """
    def handleGotNodes(self, dict):
        _krpc_sender = dict['_krpc_sender']
        dict = dict['rsp']
        sender = {'id' : dict["id"]}
        sender['port'] = _krpc_sender[1]        
        sender['host'] = _krpc_sender[0]        
        sender = self.table.Node().initWithDict(sender)
        try:
            l = unpackNodes(dict.get("nodes", []))
            if not self.answered.has_key(sender.id):
                self.answered[sender.id] = sender
        except:
            l = []
            self.table.invalidateNode(sender)
            
        if self.finished:
            # a day late and a dollar short
            return
        self.outstanding = self.outstanding - 1
        for node in l:
            n = self.table.Node().initWithDict(node)
            if not self.found.has_key(n.id):
                self.found[n.id] = n
                insort(self.foundq, NodeWrap(n, self.num))
                self.table.insertNode(n, contacted=0)
        self.schedule()
        
    def schedule(self):
        """
            send messages to new peers, if necessary
        """
        if self.finished:
            return
        l = [wrapper.node for wrapper in self.foundq[:K]]
        for node in l:
            if node.id == self.target:
                self.finished=1
                return self.callback([node])
            if self.shouldQuery(node):
                #xxxx t.timeout = time.time() + FIND_NODE_TIMEOUT
                try:
                    df = node.findNode(self.target, self.table.node.id)
                except KRPCSelfNodeError:
                    pass
                else:
                    df.addCallbacks(self.handleGotNodes, self.makeMsgFailed(node))
                    self.outstanding = self.outstanding + 1
            if self.outstanding >= const.CONCURRENT_REQS:
                break
        assert(self.outstanding) >=0
        if self.outstanding == 0:
            ## all done!!
            self.finished=1
            self._cleanup()
            self.callLater(0, self.callback, l[:K])

    def makeMsgFailed(self, node):
        return self._defaultGotNodes

    def _defaultGotNodes(self, err):
        self.outstanding = self.outstanding - 1
        self.schedule()

    def goWithNodes(self, nodes):
        """
            this starts the process, our argument is a transaction with t.extras being our list of nodes
            it's a transaction since we got called from the dispatcher
        """
        for node in nodes:
            if node.id == self.table.node.id:
                continue
            else:
                self.found[node.id] = node
                insort(self.foundq, NodeWrap(node, self.num))
        self.schedule()
    

get_value_timeout = 15
class GetValue(FindNode):
    def __init__(self, table, target, callback, callLater, find="findValue"):
        FindNode.__init__(self, table, target, callback, callLater)
        self.findValue = find
            
    """ get value task """
    def handleGotNodes(self, dict):
        _krpc_sender = dict['_krpc_sender']
        dict = dict['rsp']
        sender = {'id' : dict["id"]}
        sender['port'] = _krpc_sender[1]
        sender['host'] = _krpc_sender[0]                
        sender = self.table.Node().initWithDict(sender)
        
        if self.finished or self.answered.has_key(sender.id):
            # a day late and a dollar short
            return
        self.outstanding = self.outstanding - 1

        self.answered[sender.id] = sender
        # go through nodes
        # if we have any closer than what we already got, query them
        if dict.has_key('nodes'):
            try:
                l = unpackNodes(dict.get('nodes',[]))
            except:
                l = []
                del(self.answered[sender.id])
            
            for node in l:
                n = self.table.Node().initWithDict(node)
                if not self.found.has_key(n.id):
                    self.table.insertNode(n)
                    self.found[n.id] = n
                    insort(self.foundq, NodeWrap(n, self.num))
        elif dict.has_key('values'):
            def x(y, z=self.results):
                if not z.has_key(y):
                    z[y] = 1
                    return y
                else:
                    return None
            z = len(dict.get('values', []))
            v = filter(None, map(x, dict.get('values',[])))
            if(len(v)):
                self.callLater(0, self.callback, v)
        self.schedule()
        
    ## get value
    def schedule(self):
        if self.finished:
            return
        for node in [wrapper.node for wrapper in self.foundq[:K]]:
            if self.shouldQuery(node):
                #xxx t.timeout = time.time() + GET_VALUE_TIMEOUT
                try:
                    f = getattr(node, self.findValue)
                except AttributeError:
                    print ">>> findValue %s doesn't have a %s method!" % (node, self.findValue)
                else:
                    try:
                        df = f(self.target, self.table.node.id)
                        df.addCallback(self.handleGotNodes)
                        df.addErrback(self.makeMsgFailed(node))
                        self.outstanding = self.outstanding + 1
                        self.queried[node.id] = 1
                    except KRPCSelfNodeError:
                        pass
            if self.outstanding >= const.CONCURRENT_REQS:
                break
        assert(self.outstanding) >=0
        if self.outstanding == 0:
            ## all done, didn't find it!!
            self.finished=1
            self._cleanup()
            self.callLater(0, self.callback, [])

    ## get value
    def goWithNodes(self, nodes, found=None):
        self.results = {}
        if found:
            for n in found:
                self.results[n] = 1
        for node in nodes:
            if node.id == self.table.node.id:
                continue
            else:
                self.found[node.id] = node
                insort(self.foundq, NodeWrap(node, self.num))
        self.schedule()


class StoreValue(ActionBase):
    def __init__(self, table, target, value, callback, callLater, store="storeValue"):
        ActionBase.__init__(self, table, target, callback, callLater)
        self.value = value
        self.stored = []
        self.store = store
        
    def storedValue(self, t, node):
        self.outstanding -= 1
        if self.finished:
            return
        self.stored.append(t)
        if len(self.stored) >= const.STORE_REDUNDANCY:
            self.finished=1
            self.callback(self.stored)
        else:
            if not len(self.stored) + self.outstanding >= const.STORE_REDUNDANCY:
                self.schedule()
        return t
    
    def storeFailed(self, t, node):
        self.outstanding -= 1
        if self.finished:
            return t
        self.schedule()
        return t
    
    def schedule(self):
        if self.finished:
            return
        num = const.CONCURRENT_REQS - self.outstanding
        if num > const.STORE_REDUNDANCY - len(self.stored):
            num = const.STORE_REDUNDANCY - len(self.stored)
        if num == 0 and not self.finished:
            self.finished=1
            self.callback(self.stored)
        while num > 0:
            try:
                node = self.nodes.pop()
            except IndexError:
                if self.outstanding == 0:
                    self.finished = 1
                    self._cleanup()
                    self.callback(self.stored)
                return
            else:
                if not node.id == self.table.node.id:
                    try:
                        f = getattr(node, self.store)
                    except AttributeError:
                        print ">>> %s doesn't have a %s method!" % (node, self.store)
                    else:
                        try:
                            df = f(self.target, self.value, self.table.node.id)
                        except KRPCProtocolError:
                            self.table.table.invalidateNode(node)
                        except KRPCSelfNodeError:
                            pass
                        else:
                            df.addCallback(self.storedValue, node=node)
                            df.addErrback(self.storeFailed, node=node)
                            self.outstanding += 1
                            num -= 1
                        
    def goWithNodes(self, nodes):
        self.nodes = nodes
        self.nodes.sort(self.sort)
        self.schedule()


class GetAndStore(GetValue):
    def __init__(self, table, target, value, callback, storecallback, callLater, find="findValue", store="storeValue"):
        self.store = store
        self.value = value
        self.cb2 = callback
        self.storecallback = storecallback
        def cb(res):
            self.cb2(res)
            if not(res):
                n = StoreValue(self.table, self.target, self.value, self.doneStored, self.callLater, self.store)
                n.goWithNodes(self.answered.values())
        GetValue.__init__(self, table, target, cb, callLater, find)

    def doneStored(self, dict):
        self.storecallback(dict)
        
class KeyExpirer:
    def __init__(self, store, callLater):
        self.store = store
        self.callLater = callLater
        self.callLater(const.KEINITIAL_DELAY, self.doExpire)
    
    def doExpire(self):
        self.cut = time() - const.KE_AGE
        self.store.expire(self.cut)
        self.callLater(const.KE_DELAY, self.doExpire)



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

import khashmir, knode
from actions import *
from khash import newID
from krpc import KRPCProtocolError, KRPCFailSilently
from cache import Cache
from sha import sha
from util import *
from BitTorrent.stackthreading import Thread
from socket import gethostbyname
from const import *
from kstore import sample

TOKEN_UPDATE_INTERVAL = 5 * 60 # five minutes
NUM_PEERS = 50 # number of peers to return

class UTNode(knode.KNodeBase):
    def announcePeer(self, info_hash, port, khashmir_id):
        assert type(port) == type(1)
        assert type(info_hash) == type('')
        assert type(khashmir_id) == type('')
        assert len(info_hash) == 20
        assert len(khashmir_id) == 20

        try:
            token = self.table.tcache[self.id]
        except:
            token = None
        if token:
            assert type(token) == type("")
            assert len(token) == 20
            df = self.conn().sendRequest('announce_peer', {'info_hash':info_hash,
                                                         'port':port,
                                                         'id':khashmir_id,
                                                         'token':token})
        else:
            raise KRPCProtocolError("no write token for node")
        df.addErrback(self.errBack)
        df.addCallback(self.checkSender)
        return df
    
    def getPeers(self, info_hash, khashmir_id):
        df = self.conn().sendRequest('get_peers', {'info_hash':info_hash, 'id':khashmir_id})
        df.addErrback(self.errBack)
        df.addCallback(self.checkSender)
        return df

    def checkSender(self, dict):
        d = knode.KNodeBase.checkSender(self, dict)
        try:
            self.table.tcache[d['rsp']['id']] = d['rsp']['token']
        except KeyError:
            pass
        return d
    
class UTStoreValue(StoreValue):
    def callNode(self, node, f):
        return f(self.target, self.value, node.token, self.table.node.id)
    
class UTKhashmir(khashmir.KhashmirBase):
    _Node = UTNode

    def setup(self, host, port, data_dir, rlcount, checkpoint=True):
        khashmir.KhashmirBase.setup(self, host, port,data_dir, rlcount, checkpoint)
        self.cur_token = self.last_token = sha('')
        self.tcache = Cache()
        self.gen_token(loop=True)
        self.expire_cached_tokens(loop=True)
        
    def expire_cached_tokens(self, loop=False):
        self.tcache.expire(time() - TOKEN_UPDATE_INTERVAL)
        if loop:
            self.rawserver.external_add_task(TOKEN_UPDATE_INTERVAL,
                                             self.expire_cached_tokens, True)
                                
    def gen_token(self, loop=False):
        self.last_token = self.cur_token
        self.cur_token = sha(newID())
        if loop:
            self.rawserver.external_add_task(TOKEN_UPDATE_INTERVAL,
                                             self.gen_token, True)

    def get_token(self, host, port):
        x = self.cur_token.copy()
        x.update("%s%s" % (host, port))
        h = x.digest()
        return h

        
    def val_token(self, token, host, port):
        x = self.cur_token.copy()
        x.update("%s%s" % (host, port))
        a = x.digest()
        if token == a:
            return True

        x = self.last_token.copy()
        x.update("%s%s" % (host, port))
        b = x.digest()
        if token == b:
            return True

        return False

    def addContact(self, host, port, callback=None):
        # use dns on host, then call khashmir.addContact
        Thread(target=self._get_host, args=[host, port, callback]).start()

    def _get_host(self, host, port, callback):

        # this exception catch can go away once we actually fix the bug
        try:
            ip = gethostbyname(host)
        except TypeError, e:
            raise TypeError(str(e) + (": host(%s) port(%s)" % (repr(host), repr(port))))
        
        self.rawserver.external_add_task(0, self._got_host, ip, port, callback)

    def _got_host(self, host, port, callback):
        khashmir.KhashmirBase.addContact(self, host, port, callback)

    def announcePeer(self, info_hash, port, callback=None):
        """ stores the value for key in the global table, returns immediately, no status 
            in this implementation, peers respond but don't indicate status to storing values
            a key can have many values
        """
        def _storeValueForKey(nodes, key=info_hash, value=port, response=callback , table=self.table):
            if not response:
                # default callback
                def _storedValueHandler(sender):
                    pass
                response=_storedValueHandler
            action = UTStoreValue(self, key, value, response, self.rawserver.add_task, "announcePeer")
            self.rawserver.external_add_task(0, action.goWithNodes, nodes)
            
        # this call is asynch
        self.findNode(info_hash, _storeValueForKey)
                    
    def krpc_announce_peer(self, info_hash, port, id, token, _krpc_sender):
        sender = {'id' : id}
        sender['host'] = _krpc_sender[0]
        sender['port'] = _krpc_sender[1]
        if not self.val_token(token, sender['host'], sender['port']):
            raise KRPCProtocolError("Invalid Write Token")
        value = compact_peer_info(_krpc_sender[0], port)
        self.store[info_hash] = value
        n = self.Node().initWithDict(sender)
        self.insertNode(n, contacted=0)
        return {"id" : self.node.id}

    def retrieveValues(self, key):
        try:
            l = self.store.sample(key, NUM_PEERS)
        except KeyError:
            l = []
        return l

    def getPeers(self, info_hash, callback, searchlocal = 1):
        """ returns the values found for key in global table
            callback will be called with a list of values for each peer that returns unique values
            final callback will be an empty list - probably should change to 'more coming' arg
        """
        nodes = self.table.findNodes(info_hash, invalid=True)
        l = [x for x in nodes if x.invalid]
        if len(l) > 4:
            nodes = sample(l , 4) + self.table.findNodes(info_hash, invalid=False)[:4]
        
        # get locals
        if searchlocal:
            l = self.retrieveValues(info_hash)
            if len(l) > 0:
                self.rawserver.external_add_task(0, callback, [reducePeers(l)])
        else:
            l = []
        # create our search state
        state = GetValue(self, info_hash, callback, self.rawserver.add_task, 'getPeers')
        self.rawserver.external_add_task(0, state.goWithNodes, nodes, l)

    def getPeersAndAnnounce(self, info_hash, port, callback, searchlocal = 1):
        """ returns the values found for key in global table
            callback will be called with a list of values for each peer that returns unique values
            final callback will be an empty list - probably should change to 'more coming' arg
        """
        nodes = self.table.findNodes(info_hash, invalid=False)
        nodes += self.table.findNodes(info_hash, invalid=True)
        
        # get locals
        if searchlocal:
            l = self.retrieveValues(info_hash)
            if len(l) > 0:
                self.rawserver.external_add_task(0, callback, [reducePeers(l)])
        else:
            l = []
        # create our search state
        x = lambda a: a
        state = GetAndStore(self, info_hash, port, callback, x, self.rawserver.add_task, 'getPeers', "announcePeer")
        self.rawserver.external_add_task(0, state.goWithNodes, nodes, l)

    def krpc_get_peers(self, info_hash, id, _krpc_sender):
        sender = {'id' : id}
        sender['host'] = _krpc_sender[0]
        sender['port'] = _krpc_sender[1]        
        n = self.Node().initWithDict(sender)
        self.insertNode(n, contacted=0)
    
        l = self.retrieveValues(info_hash)
        if len(l) > 0:
            return {'values' : [reducePeers(l)],
                    "id": self.node.id,
                    "token" : self.get_token(sender['host'], sender['port'])}
        else:
            nodes = self.table.findNodes(info_hash, invalid=False)
            nodes = [node.senderDict() for node in nodes]
            return {'nodes' : packNodes(nodes),
                    "id": self.node.id,
                    "token" : self.get_token(sender['host'], sender['port'])}




import os
from BitTorrent.platform import get_sparse_files_support

# Make this a separate function because having this code in Storage.__init__()
# would make python print a SyntaxWarning (uses builtin 'file' before 'global')
def bad_libc_workaround():
    global file
    def file(name, mode = 'r', buffering = None):
        return open(name, mode)

def is_open_for_write(mode):
    if 'w' in mode:
        return True
    if 'r' in mode and '+' in mode:
        return True
    return False

    
if os.name == 'nt':
    import win32file
    FSCTL_SET_SPARSE = 0x900C4

    def _sparse_magic(handle, length=0):
        win32file.DeviceIoControl(handle, FSCTL_SET_SPARSE, '', 0, None)

        win32file.SetFilePointer(handle, length, win32file.FILE_BEGIN)
        win32file.SetEndOfFile(handle)

        win32file.SetFilePointer(handle, 0, win32file.FILE_BEGIN)

    def make_file_sparse(path, f, length=0):
        supported = get_sparse_files_support(path)
        if not supported:
            return

        handle = win32file._get_osfhandle(f.fileno())
        _sparse_magic(handle, length)

else:
    def make_file_sparse(*args, **kwargs):
        return

def open_sparse_file(path, mode, length=0, overlapped=False):
    supported = get_sparse_files_support(path)
    flags = 0
    # some day I might support sparse files elsewhere
    if not supported and os.name != 'nt':
        return file(path, mode, 0)
    
    flags = win32file.FILE_FLAG_RANDOM_ACCESS
    if overlapped:
        flags |= win32file.FILE_FLAG_OVERLAPPED
    
    # If the hFile handle is opened with the
    # FILE_FLAG_NO_BUFFERING flag set, an application can move the
    # file pointer only to sector-aligned positions.  A
    # sector-aligned position is a position that is a whole number
    # multiple of the volume sector size. An application can
    # obtain a volume sector size by calling the GetDiskFreeSpace
    # function.
    #flags |= win32file.FILE_FLAG_NO_BUFFERING

    access = win32file.GENERIC_READ
    # Shared write is necessary because lock is assigned
    # per file handle. --Dave
    share = win32file.FILE_SHARE_READ | win32file.FILE_SHARE_WRITE
    #share = win32file.FILE_SHARE_READ #| win32file.FILE_SHARE_WRITE

    if is_open_for_write(mode):
        access |= win32file.GENERIC_WRITE

    if isinstance(path, unicode):
        CreateFile = win32file.CreateFileW
    else:
        CreateFile = win32file.CreateFile

    handle = CreateFile(path, access, share, None,
                        win32file.OPEN_ALWAYS,
                        flags, None)
    
    if supported and is_open_for_write(mode):
        _sparse_magic(handle, length)
    fd = win32file._open_osfhandle(handle, os.O_BINARY)
    handle.Detach()

    f = os.fdopen(fd, mode)
    return f




# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

import unittest

import ktable, khashmir
import khash, node, knode
import actions
import test_krpc
import test_khashmir
import kstore

tests = unittest.defaultTestLoader.loadTestsFromNames(['kstore', 'khash', 'node', 'knode', 'actions',  'ktable', 'test_krpc', 'test_khashmir'])
result = unittest.TextTestRunner().run(tests)



# By Greg Hazel

debug = False

import math
import socket
import itertools
from BitTorrent.obsoletepythonsupport import set
from BitTorrent.RTTMonitor import RTTMonitor
from BitTorrent.defer import ThreadedDeferred
from BitTorrent.yielddefer import _wrap_task
from BitTorrent.HostIP import get_host_ips
from BitTorrent.platform import bttime
from BitTorrent.Lists import SizedList

class NodeFeeder(object):
    def __init__(self, external_add_task,
                 get_remote_endpoints, rttmonitor):
        self.external_add_task = external_add_task
        self.get_remote_endpoints = get_remote_endpoints
        self.rttmonitor = rttmonitor
        self.external_add_task(3, self._start)

    def _start(self):
        df = ThreadedDeferred(_wrap_task(self.external_add_task), get_host_ips, daemon=True)
        df.addCallback(self._collect_nodes)

    def _collect_nodes(self, local_ips):
        addrs = self.get_remote_endpoints()
        
        ips = set()
        for (ip, port) in addrs:
            if ip is not None and ip != "0.0.0.0" and ip not in local_ips:
                assert isinstance(ip, str)
                assert isinstance(port, int)
                ips.add(ip)
        
        self.rttmonitor.set_nodes_restart(ips)

        delay = 5
        if len(ips) > 0:
            delay = 300
    
        self.external_add_task(delay, self._collect_nodes, local_ips)


def ratio_sum_lists(a, b):
    tx = float(sum(a))
    ty = float(sum(b))
    return tx / max(ty, 0.0001)


def standard_deviation(l):
    N = len(l)
    x = 0
    
    total = float(sum(l))
    
    mean = total/N
    
    # Subtract the mean from each data item and square the difference.
    # Sum all the squared deviations.
    for i in l:
        x += (i - mean)**2.0
    
    try:
        if False:
            # Divide sum of squares by N-1 (sample variance).
            variance = x / (N - 1)
        else:
            # Divide sum of squares by N (population variance).
            variance = x / N
    except ZeroDivisionError:
        variance = 0

    stddev = math.sqrt(variance)
    
    return stddev


class BandwidthManager(object):
    
    def __init__(self, external_add_task, config, set_option,
                       get_remote_endpoints, get_rates):
        self.external_add_task = external_add_task
        self.config = config
        self.set_option = set_option
        self.get_rates = get_rates
        def got_new_rtt(rtt):
            self.external_add_task(0, self._inspect_rates, rtt)
        self.rttmonitor = RTTMonitor(got_new_rtt)
        self.nodefeeder = NodeFeeder(external_add_task=external_add_task,
                                     get_remote_endpoints=get_remote_endpoints,
                                     rttmonitor=self.rttmonitor)

        self.start_time = None

        self.max_samples = 10 # hmm...
        self.u = SizedList(self.max_samples)
        self.d = SizedList(self.max_samples)
        self.t = SizedList(self.max_samples * 10)
        self.ur = SizedList(self.max_samples)
        self.dr = SizedList(self.max_samples)

        self.current_std = 0.001        
        self.max_std = 0.001

        self.max_rates = {}
        self.max_rates["upload"] = 1.0
        self.max_rates["download"] = 1.0
        self.max_p = 1.0
        self.min_p = 2**500
        self.mid_p = ((self.max_p - self.min_p) / 2.0) + self.min_p
        self.old_p = None

    # I pulled these numbers out of my ass.

    def _method_1(self, type, t, c, old_c, rate):
        # This concept is:
        # if the correlation is high and the latency is high
        # then lower the bandwidth limit.
        # otherwise, raise it.

        if ((c > 0.96) and (t > 100)): 
            rate /= 2.0
            if debug:
                print type, "down to", rate
        else:
            rate += 500 # hmm
            if debug:
                print type, "up to", rate
        return rate
    
    def _method_2(self, type, t, c, old_c, rate):
        # This concept is:
        # if the correlation is low and the latency is high, lower the limit
        # otherwise raise it

        if ((c < 0.60) and (t > 100)): 
            rate /= 2.0
            if debug: 
                print type, "down to", rate
        else:
            rate += 500 # hmm
            if debug:
                print type, "up to", rate
        return rate

    def _method_vegasish(self, type, t, c, old_c, rate):

        middle_rtt = ((self.rttmonitor.get_min_rtt() +
                       self.rttmonitor.get_max_rtt()) / 2.0)
        if t > middle_rtt:
            rate *= 1.0/8.0
            if debug:
                print type, "down to", rate
        else:
            rate += 1000 # hmm
            if debug:
                print type, "up to", rate
        return rate            

    def _method_vegas_greg(self, type, t, c, old_c, rate):

        middle_rtt = ((self.rttmonitor.get_min_rtt() +
                       self.rttmonitor.get_max_rtt()) / 2.0)
        if t > middle_rtt and c < 0.5:
            rate *= 1.0/8.0
            if debug:
                print type, "down to", rate
        else:
            rate += 1000 # hmm
            if debug:
                print type, "up to", rate
        return rate            

    def _method_ratio(self, type, t, p, min_p, max_p, rate):
        ratio = p / max_p
        if debug:
            print "RATIO", ratio
        if ratio < 0.5:
            rate = ratio * self.max_rates[type]
            if debug:
                print type.upper(), "SET to", rate
        else:
            rate += rate * (ratio/10.0) # hmm
            if debug:
                print type.upper(), "UP to", rate
                
        return max(rate, 1000)


    def _method_stddev(self, type, std, max_std, rate):
        if std > (max_std * 0.80): # FUDGE
            rate *= 0.80 # FUDGE
            if debug:
                print type.upper(), "DOWN to", rate
        else:
            rate += 1000 # FUDGE
            if debug:
                print type.upper(), "UP to", rate

        return max(rate, 1000) # FUDGE
    
    
    def _affect_rate(self, type, std, max_std, rate, set):
        rate = self._method_stddev(type, std, max_std, rate)

        rock_bottom = False
        if rate <= 4096:
            if debug:
                print "Rock bottom"
            rock_bottom = True
            rate = 4096
    
        set(int(rate))

        return rock_bottom
        

    def _inspect_rates(self, t = None):

        if t == None:
            t = self.rttmonitor.get_current_rtt()

        if t == None:
            # this makes timeouts reduce the maximum std deviation
            self.max_std *= 0.80 # FUDGE
            return

        if self.start_time == None:
            self.start_time = bttime()
        def set_if_enabled(option, value):
            if not self.config['bandwidth_management']:
                return
            #print "Setting rate to ", value
            self.set_option(option, value)

        # TODO: slow start should be smarter than this
        if self.start_time + 20 > bttime():
            set_if_enabled('max_upload_rate', 10000000)
            set_if_enabled('max_download_rate', 10000000)

        if t < 3:
            # I simply don't believe you. Go away.
            return

        tup = self.get_rates()
        if tup == None:
            return
        u, d = tup
        #print "udt", u, d, t
        #print "uprate, downrate=", u, d

        self.u.append(u)
        self.d.append(d)
        self.t.append(t)
        self.ur.append(self.config['max_upload_rate'])
        self.dr.append(self.config['max_download_rate'])

        #s = time.time()
        #cu = correlation(self.u, self.t)
        #cd = correlation(self.d, self.t)
        #cur = correlation(self.u, self.ur)
        #cdr = correlation(self.d, self.dr)
        #e = time.time()

        self.current_std = standard_deviation(self.t)
        
        pu = ratio_sum_lists(self.u, self.t)
        pd = ratio_sum_lists(self.d, self.t)
        if len(self.u) > 2:
            lp = [ x/y for x, y in itertools.izip(self.u, self.t) ]
            min_pu = min(*lp)
            max_pu = max(*lp)
        else:
            min_pu = u / t
            max_pu = u / t
        pu = u / t

        self.max_rates["upload"] = max(max(self.u), self.max_rates["upload"])
        self.max_rates["download"] = max(max(self.d), self.max_rates["download"])

        if debug:
            print "STDDEV", u, self.config['max_upload_rate'], \
                  self.max_std, self.current_std, pu, pd
        
        rb = self._affect_rate("upload", self.current_std, self.max_std,
                               self.config['max_upload_rate'],
                               lambda r : set_if_enabled('max_upload_rate', r))
        # don't adjust download rate, it's not nearly correlated enough
##        if rb:
##            v = int(self.config['max_download_rate'] * 0.90) # FUDGE
##            v = max(v, 2000) # FUDGE
##            set_if_enabled('max_download_rate', v) 
##        else:
##            v = int(self.config['max_download_rate'] + 6000) # FUDGE
##            set_if_enabled('max_download_rate', v) 
##        if debug:
##            print "DOWNLOAD SET to", v
            
        #self._affect_rate("download", t, cd, self.last_cd, pd,
        #                  self.config['max_download_rate'],
        #                  lambda r : self.set_option('max_download_rate', r))

        self.max_std = max(self.max_std, self.current_std)

        #self.last_cu = cu
        #self.last_cd = cd

        # we're re-called by the pinging thing
        #self.external_add_task(0.1, self._inspect_rates)
        
        


# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.
#
# Written by Matt Chisholm
# Client list updated by Ed Savage-Jones - May 28th 2005

import re

v64p = '[\da-zA-Z.-]{3}'

matches = (
           ('-AZ(?P<version>\d+)-+.+$'       , "Azureus"              ),
           ('M(?P<version>\d-\d-\d)--.+$'    , "BitTorrent"           ),
           ('T(?P<version>%s)0?-+.+$'%v64p   , "BitTornado"           ),
           ('-UT(?P<version>[\dA-F]+)-+.+$'  , u"\xb5Torrent"         ),
           ('-TS(?P<version>\d+)-+.+$'       , "TorrentStorm"         ),
           ('exbc(?P<bcver>.+)LORD.+$'       , "BitLord"              ),
           ('exbc(?P<bcver>[^-][^-]+)(?!---).+$', "BitComet"          ),
           ('-BC0(?P<version>\d+)-.+$'       , "BitComet"             ),
           ('FUTB(?P<bcver>.+).+$'           , "BitComet Mod1"        ),
           ('xUTB(?P<bcver>.+).+$'           , "BitComet Mod2"        ),
           ('A(?P<version>%s)-+.+$'%v64p     , "ABC"                  ),
           ('S(?P<version>%s)-+.+$'%v64p     , "Shadow's"             ),
           (chr(0)*12 + 'aa.+$'              , "Experimental 3.2.1b2" ),
           (chr(0)*12 + '.+$'                , "BitTorrent (obsolete)"),
           ('-G3.+$'                         , "G3Torrent"            ),
           ('-[Ll][Tt](?P<version>\d+)-+.+$' , "libtorrent"           ),
           ('Mbrst(?P<version>\d-\d-\d).+$'  , "burst!"               ),
           ('eX.+$'                          , "eXeem"                ),
           ('\x00\x02BS.+(?P<strver>UDP0|HTTPBT)$', "BitSpirit v2"    ),
           ('\x00[\x02|\x00]BS.+$'           , "BitSpirit v2"         ),
           ('.*(?P<strver>UDP0|HTTPBT)$'     , "BitSpirit"            ),
           ('-BOWP?(?P<version>[\dA-F]+)-.+$', "Bits on Wheels"       ),
           ('(?P<rsver>.+)RSAnonymous.+$'    , "Rufus Anonymous"      ),
           ('(?P<rsver>.+)RS.+$'             , "Rufus"                ),
           ('-ML(?P<version>(\d\.)+\d)(?:\.(?P<strver>CVS))?-+.+$',"MLDonkey"),
           ('346------.+$'                   , "TorrentTopia 1.70"    ),
           ('OP(?P<strver>\d{4}).+$'         , "Opera"                ),
           ('-KT(?P<version>\d+)(?P<rc>R\d+)-+.+$', "KTorrent"        ),
# Unknown but seen in peer lists:
           ('-S(?P<version>10059)-+.+$'      , "S (unknown)"          ),
           ('-TR(?P<version>\d+)-+.+$'       , "transmission"         ),
           ('S\x05\x07\x06\x00{7}.+'         , "S 576 (unknown)"      ),
# Clients I've never actually seen in a peer list:           
           ('exbc..---.+$'                   , "BitVampire 1.3.1"     ),
           ('-BB(?P<version>\d+)-+.+$'       , "BitBuddy"             ),
           ('-CT(?P<version>\d+)-+.+$'       , "CTorrent"             ),
           ('-MT(?P<version>\d+)-+.+$'       , "MoonlightTorrent"     ),
           ('-BX(?P<version>\d+)-+.+$'       , "BitTorrent X"         ),
           ('-TN(?P<version>\d+)-+.+$'       , "TorrentDotNET"        ),
           ('-SS(?P<version>\d+)-+.+$'       , "SwarmScope"           ),
           ('-XT(?P<version>\d+)-+.+$'       , "XanTorrent"           ),
           ('U(?P<version>\d+)-+.+$'         , "UPnP NAT Bit Torrent" ),
           ('-AR(?P<version>\d+)-+.+$'       , "Arctic"               ),
           ('(?P<rsver>.+)BM.+$'             , "BitMagnet"            ),
           ('BG(?P<version>\d+).+$'          , "BTGetit"              ),
           ('-eX(?P<version>[\dA-Fa-f]+)-.+$',"eXeem beta"            ),
           ('Plus12(?P<rc>[\dR]+)-.+$'       , "Plus! II"             ),
           ('XBT(?P<version>\d+)[d-]-.+$'    , "XBT"                  ),
           ('-ZT(?P<version>\d+)-+.+$'       , "ZipTorrent"           ),
           ('-BitE\?(?P<version>\d+)-.+$'    , "BitEruct"             ),
           ('O(?P<version>%s)-+.+$'%v64p     , "Osprey Permaseed"     ),
# Guesses based on Rufus source code, never seen in the wild:
           ('-BS(?P<version>\d+)-+.+$'       , "BTSlave"              ),
           ('-SB(?P<version>\d+)-+.+$'       , "SwiftBit"             ),
           ('-SN(?P<version>\d+)-+.+$'       , "ShareNET"             ),
           ('-bk(?P<version>\d+)-+.+$'       , "BitKitten"            ),
           ('-SZ(?P<version>\d+)-+.+$'       , "Shareaza"             ),
           ('-MP(?P<version>\d+)-+.+$'       , "MooPolice"            ),
           ('Deadman Walking-.+$'            , "Deadman"              ),
           ('270------.+$'                   , "GreedBT 2.7.0"        ),
           ('XTORR302.+$'                    , "TorrenTres 0.0.2"     ),
           ('turbobt(?P<version>\d\.\d).+$'  , "TurboBT"              ),
           ('DansClient.+$'                  , "XanTorrent"           ),
           ('-PO(?P<version>\d+)-+.+$'       , "PO (unknown)"         ),
           ('-UR(?P<version>\d+)-+.+$'       , "UR (unknown)"         ),
# Patterns that should be executed last
           ('.*Azureus.*'                    , "Azureus 2.0.3.2"      ),
           )

matches = [(re.compile(pattern, re.DOTALL), name) for pattern, name in matches]

unknown_clients = {}

def identify_client(peerid, log=None):
    client = 'unknown'
    version = ''
    for pat, name in matches:
        m = pat.match(peerid)
        if m:
            client = name
            d = m.groupdict()
            if d.has_key('version'):
                version = d['version']
                version = version.replace('-','.')
                if version.find('.') >= 0:
                    version = ''.join(version.split('.'))

                version = list(version)
                for i,c in enumerate(version):
                    if '0' <= c <= '9':
                        version[i] = c
                    elif 'A' <= c <= 'Z':
                        version[i] = str(ord(c) - 55)
                    elif 'a' <= c <= 'z':
                        version[i] = str(ord(c) - 61)
                    elif c == '.':
                        version[i] = '62'
                    elif c == '-':
                        version[i] = '63'
                    else:
                        break
                version = '.'.join(version)
            elif d.has_key('bcver'):
                bcver = d['bcver']
                version += str(ord(bcver[0])) + '.'
                if len(bcver) > 1:
                    version += str(ord(bcver[1])/10)
                    version += str(ord(bcver[1])%10)
            elif d.has_key('rsver'):
                rsver = d['rsver']
                version += str(ord(rsver[0])) + '.'
                if len(rsver) > 1:
                    version += str(ord(rsver[1])/10) + '.'
                    version += str(ord(rsver[1])%10)
            if d.has_key('strver'):
                if d['strver'] is not None:
                    version += d['strver']
            if d.has_key('rc'):
                rc = 'RC ' + d['rc'][1:]
                if version:
                    version += ' '
                version += rc
            break
    if client == 'unknown':
        # identify Shareaza 2.0 - 2.1
        if len(peerid) == 20 and chr(0) not in peerid[:15]:
            for i in range(16,20):
                 if ord(peerid[i]) != (ord(peerid[i - 16]) ^ ord(peerid[31 - i])):
                     break
            else:
                client = "Shareaza"
        
        
    if log is not None and 'unknown' in client:
        if not unknown_clients.has_key(peerid):
            unknown_clients[peerid] = True
            log.write('%s\n'%peerid)
            log.write('------------------------------\n')
    return client, version



#!/usr/bin/env python

# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Bram Cohen and Matt Chisholm

import os

from BitTorrent.platform import install_translation
install_translation()

import sys
from distutils.core import setup, Extension
from BitTorrent import version, languages
from BitTorrent.platform import calc_unix_dirs

import glob

# detect case-insensitive filesystem
case_sensitive_filesystem = True
os.mkdir('FOO')
try:
    f = open('foo', 'w')
except:
    case_sensitive_filesystem = False
else:
    f.close()
    os.remove('foo')
os.rmdir('FOO')
# done detecting case-insensitive filesystem

extra_docs = []

symlinks = ["bittorrent" , "bittorrent-curses", "bittorrent-console",
           "maketorrent",                      "maketorrent-console",
                          "launchmany-curses", "launchmany-console",
                                               "changetracker-console",
                                               "torrentinfo-console",
           "bittorrent-tracker",
           ]

scripts = []

for s in symlinks:
    script = s+'.py'
    if not os.access(s, os.F_OK):
        os.symlink(script, s)
    scripts.append(script)
    os.chmod(script, 0755)

use_scripts = symlinks
if sys.argv[1:2] == ['sdist'] or not case_sensitive_filesystem:
    use_scripts = scripts
    extra_docs.append('BUILD.windows.txt')

img_root, doc_root, locale_root = calc_unix_dirs()


data_files = [
    (img_root, ['images/bittorrent.ico',]),
    (doc_root, ['credits.txt', 'credits-l10n.txt', 'LICENSE.txt', 'README.txt',
                'TRACKERLESS.txt', 'redirdonate.html', 'public.key',
                'INSTALL.unix.txt'] + extra_docs),
    ]

for d in ('flags', 'logo', 'themes/default',
          'themes/default/statuslight', 'themes/default/torrentstate',
          'themes/default/torrentops' , 'themes/default/fileops'     ,):
    data_files.append(
        (os.path.join(img_root, d),
         glob.glob(os.path.join('images', d, '*.png')))
        )

if not os.path.exists('locale'):
    os.system('sh ./makei18n.sh')

for l in languages:
    path = os.path.join('locale', l, 'LC_MESSAGES', 'bittorrent.mo')
    if os.access(path, os.F_OK):
        data_files.append((os.path.join(locale_root, l, 'LC_MESSAGES'), 
                             [path,]))

attrs = {
    'name' : "BitTorrent",
    'version' : version,
    'author' : "Bram Cohen",
    'author_email' : "bugs@bittorrent.com",
    'url' : "http://bittorrent.com/",
    'license' : "BitTorrent Open Source License",
    'scripts' : use_scripts,
    'packages' : ["BitTorrent", "khashmir", "BitTorrent.GUI_wx",],
    'py_modules' : ["Zeroconf",],
    'data_files' : data_files,
    'description' : "Scatter-gather network file transfer",
    'long_description' : """BitTorrent is a tool for distributing files.  It's extremely easy to use - downloads are started by clicking on hyperlinks.  Whenever more than one person is downloading at once they send pieces of the file(s) to each other, thus relieving the central server's bandwidth burden.  Even with many simultaneous downloads, the upload burden on the central server remains quite small, since each new downloader introduces new upload capacity.""",
}

setup(**attrs)

for s in symlinks:
    if os.path.islink(s):
        os.remove(s)



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

import unittest
from BitTorrent.platform import bttime
from time import sleep

from kstore import KStore
if __name__ =="__main__":
    tests = unittest.defaultTestLoader.loadTestsFromNames(['test_kstore'])
    result = unittest.TextTestRunner().run(tests)


class BasicTests(unittest.TestCase):
    def setUp(self):
        self.k = KStore()
        
    def testNoKeys(self):
        self.assertEqual(self.k.keys(), [])

    def testKey(self):
        self.k['foo'] = 'bar'
        self.assertEqual(self.k.keys(), ['foo'])

    def testKeys(self):
        self.k['foo'] = 'bar'
        self.k['wing'] = 'wang'
        l = self.k.keys()
        l.sort()
        self.assertEqual(l, ['foo', 'wing'])
        
    def testInsert(self):
        self.k['foo'] = 'bar'
        self.assertEqual(self.k['foo'], ['bar'])

    def testInsertTwo(self):
        self.k['foo'] = 'bar'
        self.k['foo'] = 'bing'
        l = self.k['foo']
        l.sort()
        self.assertEqual(l, ['bar', 'bing'])
        
    def testExpire(self):
        self.k['foo'] = 'bar'
        self.k.expire(bttime() - 1)
        l = self.k['foo']
        l.sort()
        self.assertEqual(l, ['bar'])
        self.k['foo'] = 'bing'
        t = bttime()
        self.k.expire(bttime() - 1)
        l = self.k['foo']
        l.sort()
        self.assertEqual(l, ['bar', 'bing'])        
        self.k['foo'] = 'ding'
        self.k['foo'] = 'dang'
        l = self.k['foo']
        l.sort()
        self.assertEqual(l, ['bar', 'bing', 'dang', 'ding'])
        self.k.expire(t)
        l = self.k['foo']
        l.sort()
        self.assertEqual(l, ['dang', 'ding'])
        
    def testDup(self):
        self.k['foo'] = 'bar'
        self.k['foo'] = 'bar'
        self.assertEqual(self.k['foo'], ['bar'])

    def testSample(self):
        for i in xrange(2):
            self.k['foo'] = i
        l = self.k.sample('foo', 5)
        l.sort()
        self.assertEqual(l, [0, 1])

        for i in xrange(10):
            for i in xrange(10):
                self.k['bar'] = i
            l = self.k.sample('bar', 5)
            self.assertEqual(len(l), 5)
            for i in xrange(len(l)):
                self.assert_(l[i] not in l[i+1:])
        



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Author: David Harrison

if __name__ == "__main__":
    def _(s):
        return s
else:
    from BitTorrent.translation import _

import os

if os.name == 'nt':
    import win32file, win32event, win32api, winerror
    from win32file import CreateFile, CreateFileW
    import pywintypes

elif os.name == 'posix':
    import fcntl
    from BitTorrent import platform
    from fcntl import flock


class NamedMutex(object):
    """Reasonably cross-platform, cross-process mutex.

       It does not implement mutual exclusion between threads in the
       same process.  Use threading.Lock or threading.RLock for mutual
       exclusion between threads.
    """
    # In Unix:
    #   Same semantics as directly locking a file: mutual exclusion is
    #   provided when a process can lock the file.  If
    #   the file does not exist, it is created. 
    #
    # In NT:
    #   Uses Windows CreateMutex.
    # obtain_mutex = 1
    # mutex = win32event.CreateMutex(None, obtain_mutex, name)

    def __init__(self, name):
        self._mylock = False
        self._name = name
        self._mutex = None 
        if os.name in ('posix','max'):
            # We don't use config["data_dir"] to keep mutex separate from 
            # other configuration data since bittorrent.py will have a 
            # separate config directory from bittorrent console and curses 
            # apps.
            ddir = platform.get_dot_dir()
            self._path = os.path.join( ddir, "mutex", name )
            if not os.path.exists(ddir):
                os.mkdir(ddir, 0700)

    def owner(self):
        return self._mylock
    
    def acquire(self, wait=True):  
        """Acquires mutual exclusion.  Returns true iff acquired."""
        if os.name == 'nt':

            # Gotcha: Not checking self._mylock in windows, because it is
            # possible to acquire the mutex from more than one object with
            # the same name.  This needs some work to make the semantics
            # exactly the same between Windows and Unix.
            obtain_mutex = 1
            self._mutex = win32event.CreateMutex(None,obtain_mutex,self._name)
            if self._mutex is None:
                return False
            lasterror = win32api.GetLastError()
            if lasterror == winerror.ERROR_ALREADY_EXISTS:
                if wait:
                   # mutex exists and has been opened(not created, not locked).
                   r = win32event.WaitForSingleObject(self._mutex, 
                                                      win32event.INFINITE)
                else:
                   r = win32event.WaitForSingleObject(self._mutex, 0)

                # WAIT_OBJECT_0 means the mutex was obtained
                # WAIT_ABANDONED means the mutex was obtained, 
                # and it had previously been abandoned
                if (r != win32event.WAIT_OBJECT_0 and 
                    r != win32event.WAIT_ABANDONED):
                    return False
    
        elif os.name in ('posix','max'):
            if self._mylock: 
                return True
            
            if os.path.exists(self._path) and not os.path.isfile(self._path):
                raise BTFailure( 
                    "Cannot lock file that is not regular file." )

            (dir,name) = os.path.split(self._path)
            if not os.path.exists(dir):
                os.mkdir(dir, 0700)  # mode=0700 = allow user access.

            while True:  # <--- for file deletion race condition (see __del__)

                # UNIX does not support O_TEMPORARY!! Blech. This complicates
                # file deletion (see "while True" above and path checking after
                # "flock").
                #self._mutex = os.open( self._path, os.O_CREAT|os.O_TEMPORARY )
                self._mutex = open( self._path, "w" )
                if wait:
                    flags = fcntl.LOCK_EX
                else:
                    flags = fcntl.LOCK_EX | fcntl.LOCK_NB
                try:
                    flock( self._mutex.fileno(), flags)
                except IOError:
                    return False

                # race condition: __del__ may have deleted the file.
                if not os.path.exists( self._path ):
                    self._mutex.close()
                else:
                    break
    
        else:
            # dangerous, but what should we do if the platform neither
            # supports named mutexes nor file locking?   --Dave
            pass
            
        self._mylock = True
        return True
  
    def release(self):
        # unlock
        assert self._mylock
        if os.name == 'nt':
            win32event.ReleaseMutex(self._mutex)
            # Error code 123? 
            #lasterror = win32api.GetLastError()
            #if lasterror != 0:
            #    raise IOError( _("Could not release mutex %s due to "
            #                     "error windows code %d.") % 
            #                    (self._name,lasterror) )
        elif os.name == 'posix':
            self._mylock = False
            if not os.path.exists(self._path):
                raise IOError( _("Non-existent file: %s") % self._path )
            flock( self._mutex.fileno(), fcntl.LOCK_UN )
            self._mutex.close()

    def __del__(self):

        if os.name == 'nt':
            if self._mutex is not None:
                # Gotchas: Don't close the handle before releasing it or the
                # mutex won't release until the python script exits.  It's 
                # safe to call ReleaseMutex even if the local process hasn't 
                # acquired the mutex.  If this process doesn't have mutex then 
                # the call fails.  Note that in Windows, mutexes are literally 
                # per process.  Multiple mutexes created with the same name
                # from the same process will be treated as one with respect to 
                # release and acquire.
                self.release()

                # windows will destroy the mutex when the last handle to that
                # mutex is closed.
                win32api.CloseHandle(self._mutex)
                del self._mutex
   
        elif os.name == 'posix':
            if self._mylock:
                self.release()

            # relock file non-blocking to see if anyone else was 
            # waiting on it.  (A race condition exists where another process
            # could have just opened but not yet locked the file.  This process
            # then deletes the file.  When the other process resumes, the
            # flock call fails.  This is however not a particularly bad
            # race condition since acquire() simply repeats the open & flock.)
            if os.path.exists(self._path) and self.acquire(False):

                try:
                    os.remove(self._path)
                except:
                    pass

                # Strange to unlock the file after it has been deleted.
                # Works on all UNIX platforms?  If I release before
                # deleting then I introduce a race condition where the
                # another process acquires the lock then
                # this process deletes the file.
                flock( self._mutex.fileno(), fcntl.LOCK_UN )
                self._mutex.close()

if __name__ == "__main__":
    # perform unit tests.
    n_tests = n_tests_passed = 0
    n_tests += 1
    mutex = NamedMutex("blah")    
    if mutex.acquire() and mutex._mutex is not None and mutex._mylock:
        n_tests_passed += 1
    else:
        print "FAIL! Failed to acquire mutex on a new NamedMutex."

    # attempt to acquire again.  Since already acquired, it should fail.
    #n_tests += 1
    #if not mutex.acquire():
    #    n_tests_passed += 1
    #else:
    #    print "FAIL! Second acquire should return false but returned true."

    n_tests += 1
    mutex.release()
    if mutex._mutex is not None and not mutex._mylock:
        n_tests_passed += 1
    else:
        print "FAIL! Did not properly release mutex."

    n_tests += 1
    if mutex.acquire():
        if mutex._mutex is not None and mutex._mylock:
            n_tests_passed += 1
        else: 
            print ( "FAIL! After calling acquire on a released NamedMutex, "
                    "either mutex._mutex is None or mutex._mylock is false." )
    else:
        print "FAIL! Failed to acquire mutex a released NamedMutex."


    # okay.  I should add more tests.

    del mutex

    if n_tests == n_tests_passed:
        print "Passed all %d tests." % n_tests



import sys
from BitTorrent import language_names

if '-a' in sys.argv:
    from BitTorrent.language import unfinished_language_names
    language_names.update(unfinished_language_names)

languages = language_names.keys()
languages.sort()

print ' '.join(languages)



import win32gui
TTS_BALLOON = 0x40
WM_USER = 1024
WM_TRAYMESSAGE = WM_USER + 20

def _get_nid(hwnd, id, flags, callbackmessage, hicon, title, msg):
        nid = (hwnd, id, flags, callbackmessage, hicon)
        nid = list(nid)

        nid.append('') # the tip
        nid.append(msg) # the balloon message
        nid.append(15000) # the timeout
        nid.append(title) # the title
        nid.append(win32gui.NIIF_INFO) # also warning and error available

        return tuple(nid)

_hwnd = None
def find_traywindow_hwnd():
    global _hwnd
    if _hwnd is None:
        try:
            _hwnd = win32gui.FindWindowEx(0, 0, 'wxWindowClassNR', '')
        except:
            pass
    return _hwnd

def SetBalloonTip(hicon, title, msg):
    hwnd = find_traywindow_hwnd()
    id = 99 # always 99
    flags = win32gui.NIF_MESSAGE | win32gui.NIF_ICON | win32gui.NIF_INFO
    callbackmessage = WM_TRAYMESSAGE
    nid = _get_nid(hwnd, id, flags, callbackmessage, hicon, title, msg)
    try:
        win32gui.Shell_NotifyIcon(win32gui.NIM_MODIFY, nid)
    except:
        pass



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.
#
# Written by Matt Chisholm

import libxml2
import BitTorrent.stackthreading as threading
#import gobject
from PluginSupport import PluginManager, BasePlugin

# BUG should be in config
UPDATE_FEED_TIMEOUT = 5 * 60 

def get_content(xpath_result):
    if len(xpath_result) == 1:
        return xpath_result[0].content
    return None

class FeedPlugin(BasePlugin):
    mimetype = 'text/xml'
    subtype = None
    
    def __init__(self, main, url, title, description, doc=None):
        BasePlugin.__init__(self, main)
        self.url = url
        self.title = title
        self.description = description
        self.doc = None
        self.thread = None
        self.update(doc)
        timeout_id = gobject.timeout_add(UPDATE_FEED_TIMEOUT * 1000, self.update)

    def _matches_type(mimetype, subtype):
        raise NotImplementedError

    matches_type = staticmethod(_matches_type)

    def update(self, doc=None):
        if self.thread is not None:
            if self.thread.isAlive():
                return True
        
        self.thread = threading.Thread(target=self._update,
                                       args=(doc,))
        self.thread.setDaemon(True)
        self.main.show_status("Downloading %s" % self.url)
        self.main.ui_wrap_func(self.thread.start)
        return True
        

    def _update(self, doc=None):
        self.main.show_status('_update running')
        if self.doc is not None:
            self.main.show_status('_update freeing doc')
            self.doc.freeDoc()
        if doc is None:
            self.main.show_status('_update parsingFile')
            doc = libxml2.parseFile(self.url)
        self.doc = doc
        self.main.feed_was_updated(self.url)
        self.main.show_status(self.get_items())
        self.main.show_status('_update done, thread should die')


    def get_items(self):
        raise NotImplementedError


class FeedManager(PluginManager):
    kind = 'Feed'

    def __init__(self, config, ui_wrap_func):
        PluginManager.__init__(self, config, ui_wrap_func)
        self.ui_wrap_func = ui_wrap_func
        self.channels = {}
        self.update_all()


    def _check_plugin(self, plugin):
        if not PluginManager._check_plugin(self, plugin):
            return False
        if issubclass(plugin, FeedPlugin):
            if hasattr(plugin, 'get_items'):
                return True
        return False 


    def update_all(self):
        for k in self.config.keys():
            self.update_feed(k)


    def update_feed(self, feed_url):
        if self.channels.has_key(feed_url):
            self.show_status("Updating %s" % feed_url)
            self.channels[feed_url].update()
        else:
            self.new_channel(feed_url)
        return

    def new_channel(self, url):
        thread = threading.Thread(target=self._new_channel,
                                       args=(url,))
        thread.setDaemon(True)
        self.ui_wrap_func(thread.start)
        

    def _new_channel(self, url):
        self.show_status("Adding %s" % url)
        doc = None
        channel = None
        try:
            doc = libxml2.parseFile(url)
        except libxml2.parserError:
            self.show_status("Could not parse \"%s\" as XML, searching for torrents" % url)
        else:
            context = doc.xpathNewContext()
            res = context.xpathEval('/rss/@version')
            if get_content(res) in ('0.91', '0.92', '2.0'):
                self.show_status("Found RSS 0.9x/2.0")
                plugin = self._find_plugin('text/xml', 'rss2')
                if plugin is not None:
                    channel = plugin(self, url, doc)
            
            root = doc.children
            if (root.ns() is not None and
                root.ns().get_content() in ("http://www.w3.org/2005/Atom",)):
                self.show_status("Found Atom")
                plugin = self._find_plugin('text/xml', 'atom')
                if plugin is not None:
                    channel = plugin(self, url, doc)
                
            context.xpathFreeContext()
        if channel is None:
            if doc is not None:
                doc.freeDoc()
            self.show_status("Unknown feed type, using raw")
            plugin = self._find_plugin(None, 'raw')
            if plugin is not None:
                channel = plugin(self, url, url)
            
        self.channels[url] = channel


    def feed_was_updated(self, feed_url):
        self.run_ui_task('feed_was_updated', feed_url)


    def get_feed(self, feed_url):
        if not self.channels.has_key(feed_url):
            self.update_feed(feed_url)
        feed = self.channels[feed_url]
        return feed

    def get_items(self, feed_url):
        channel = self.channels[feed_url]
        items = channel.get_items()
        return items

        
    



#!/usr/bin/env python

# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Bram Cohen

if __name__ == '__main__':
    from BitTorrent.translation import _
    
import os
from sys import argv
from BitTorrent.track import track

if __name__ == '__main__':
    track(argv[1:])



# QList:
# basically a python 2.3 compatible interface if you want deque
#
# SizedList:
# handy class for keeping a fixed-length history
# uses deque if available
#
# by Greg Hazel

try:
    from collections import deque
    base_list_class = deque
    popleft = deque.popleft
    clear = deque.clear
except ImportError:
    from UserList import UserList
    base_list_class = UserList
    def popleft(l):
        return l.pop(0)
    def clear(l):
        l[:] = []

class QList(base_list_class):

    def __init__(self, *a, **kw):
        base_list_class.__init__(self, *a, **kw)

    def popleft(self):
        return popleft(self)

    def clear(self):
        return clear(self)

    # dequeu doesn't have __add__ ?
    def __add__(self, l):
        n = base_list_class(self)
        n.extend(l)
        return n

# I use QList becuase deque.popleft is faster than list.pop(0)
class SizedList(QList):

    def __init__(self, max_items):
        self.max_items = max_items
        QList.__init__(self)

    def append(self, v):
        QList.append(self, v)
        if len(self) > self.max_items:
            self.popleft()        
            
if __name__ == '__main__':
    l = SizedList(10)
    for i in xrange(50):
        l.append(i)
    assert list(l) == range(40, 50)    


# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Bram Cohen

from BitTorrent.platform import bttime


class Measure(object):

    def __init__(self, max_rate_period, fudge=5):
        self.max_rate_period = max_rate_period
        self.ratesince = bttime() - fudge
        self.last = self.ratesince
        self.rate = 0.0
        self.total = 0
        self.when_next_expected = bttime() + fudge

    def update_rate(self, amount):
        self.total += amount
        t = bttime()
        if t < self.when_next_expected and amount == 0:
            return self.rate
        self.rate = (self.rate * (self.last - self.ratesince) +
                     amount) / (t - self.ratesince)
        self.last = t
        self.ratesince = max(self.ratesince, t - self.max_rate_period)
        self.when_next_expected = t + min((amount / max(self.rate, 0.0001)), 5)

    def get_rate(self):
        self.update_rate(0)
        return self.rate

    def get_rate_noupdate(self):
        return self.rate

    def time_until_rate(self, newrate):
        if self.rate <= newrate:
            return 0
        t = bttime() - self.ratesince
        return ((self.rate * t) / newrate) - t

    def get_total(self):
        return self.total



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Greg Hazel

import os
import sys
import socket
import signal
import string
import struct
import thread
import logging
import threading
import traceback

from DictWithLists import DictWithLists

from BitTorrent.defer import DeferredEvent, Deferred, run_deferred
from BitTorrent.translation import _
from BitTorrent import BTFailure
from BitTorrent.obsoletepythonsupport import set


##############################################################
#profile = True
profile = False

if profile:
    try:
        import hotshot
        import hotshot.stats
        prof_file_name = 'rawserver.prof'
    except ImportError, e:
        print "profiling not available:", e
        profile = False
##############################################################

from twisted.python import threadable
# needed for twisted 1.3
# otherwise the 'thread safety' functions are not 'thread safe'
threadable.init(1)

letters = set(string.letters)

noSignals = True

if os.name == 'nt':
    try:
        from twisted.internet import iocpreactor
        iocpreactor.proactor.install()
        noSignals = False
    except:
        # just as limited (if not more) as select, and also buggy
        #try:
        #    from twisted.internet import win32eventreactor
        #    win32eventreactor.install()
        #except:
        #    pass
        pass
else:
    try:
        from twisted.internet import kqreactor
        kqreactor.install()
    except:
        try:
            from twisted.internet import pollreactor
            pollreactor.install()
        except:
            pass

rawserver_logger = logging.getLogger('RawServer')

#the default reactor is select-based, and will be install()ed if another has not
from twisted.internet import reactor, task, error

# as far as I know, we work with twisted 1.3 and >= 2.0
#import twisted.copyright
#if twisted.copyright.version.split('.') < 2:
#    raise ImportError(_("RawServer_twisted requires twisted 2.0.0 or greater"))

from twisted.internet.protocol import DatagramProtocol, Protocol, ClientFactory
from twisted.protocols.policies import TimeoutMixin
from twisted.internet import interfaces, address

from BitTorrent.ConnectionRateLimitReactor import connectionRateLimitReactor

NOLINGER = struct.pack('ii', 1, 0)

# python sucks.
SHUT_RD = getattr(socket, 'SHUT_RD', 0)
SHUT_WR = getattr(socket, 'SHUT_WR', 1)

# this is a base class for all the callbacks the server could use
class Handler(object):

    # there is no connection_started.
    # a connection request can result in either connection_failed or
    # connection_made

    # called when the connection is ready for writiing
    def connection_made(self, s):
        pass

    # called when a connection request failed (failed, refused, or requested to close)
    def connection_failed(self, addr, exception):
        pass

    def data_came_in(self, addr, data):
        pass

    # called once when the current write buffer empties completely
    def connection_flushed(self, s):
        pass

    # called when a connection dies (lost or requested to close)
    def connection_lost(self, s):
        pass


class ConnectionWrapper(object):

    def __init__(self, rawserver, handler, context, tos=0):
        self.ip = None             # peer ip
        self.tos = tos
        self.port = None           # peer port
        self.dying = False
        self.paused = False
        self.encrypt = None
        self.connector = None
        self.transport = None
        self.reset_timeout = None
        self.callback_connection = None

        self.buffer = OutputBuffer(self._flushed)

        self.post_init(rawserver, handler, context)

    def post_init(self, rawserver, handler, context):
        self.rawserver = rawserver
        self.handler = handler
        self.context = context
        if self.rawserver:
            self.rawserver.single_sockets.add(self)

    def get_socket(self):
        s = None
        try:
            s = self.transport.getHandle()
        except:
            try:
                # iocpreactor doesn't implement ISystemHandle like it should
                s = self.transport.socket
            except:
                pass
        return s

    def pause_reading(self):
        # interfaces are the stupedist crap ever
        if (hasattr(interfaces.IProducer, "providedBy") and
            not interfaces.IProducer.providedBy(self.transport)):
            print "No producer", self.ip, self.port, self.transport
            return
        # not explicitly needed, but iocpreactor has a bug where the author is a moron
        if self.paused:
            return
        self.transport.pauseProducing()
        self.paused = True

    def resume_reading(self):
        if (hasattr(interfaces.IProducer, "providedBy") and
            not interfaces.IProducer.providedBy(self.transport)):
            print "No producer", self.ip, self.port, self.transport
            return
        # not explicitly needed, but iocpreactor has a bug where the author is a moron
        if not self.paused:
            return
        self.paused = False
        try:
            self.transport.resumeProducing()
        except Exception, e:
            # I bet these are harmless
            print "resumeProducing error", type(e), e

    def attach_transport(self, callback_connection, transport, reset_timeout):
        self.transport = transport
        self.callback_connection = callback_connection
        self.reset_timeout = reset_timeout

        if hasattr(transport, 'addBufferCallback'):
            self.buffer = PassBuffer(self.transport, self._flushed, self.buffer)
        else:
            self.buffer.attachConsumer(self.transport)

        try:
            address = self.transport.getPeer()
        except:
            # udp, for example
            address = self.transport.getHost()

        try:
            self.ip = address.host
            self.port = address.port
        except:
            # unix sockets, for example
            pass

        if self.tos != 0:
            s = self.get_socket()

            try:
                s.setsockopt(socket.IPPROTO_IP, socket.IP_TOS, self.tos)
            except socket.error:
                pass

    def sendto(self, packet, flags, addr):
        ret = None
        try:
            ret = self.transport.write(packet, addr)
        except Exception, e:
            rawserver_logger.warning("UDP sendto failed: %s" % unicode(e.args[0]))

        return ret

    def write(self, b):
        if self.encrypt is not None:
            b = self.encrypt(b)
        # bleh
        if isinstance(b, buffer):
            b = str(b)
        self.buffer.add(b)

    def _flushed(self):
        s = self
        # why do you tease me so?
        if s.handler is not None:
            # calling flushed from the write is bad form
            self.rawserver.add_task(0, s.handler.connection_flushed, s)

    def is_flushed(self):
        return self.buffer.is_flushed()

    def shutdown(self, how):
        if how == SHUT_WR:
            if hasattr(self.transport, "loseWriteConnection"):
                self.transport.loseWriteConnection()
            else:
                # twisted 1.3 sucks
                try:
                    self.transport.socket.shutdown(how)
                except:
                    pass
            self.buffer.stopWriting()
        elif how == SHUT_RD:
            self.transport.stopListening()
        else:
            self.close()

    def close(self):
        self.buffer.stopWriting()

        if self.rawserver.config['close_with_rst']:
            try:
                s = self.get_socket()
                s.setsockopt(socket.SOL_SOCKET, socket.SO_LINGER, NOLINGER)
            except:
                pass

        if self.transport:
            if self in self.rawserver.udp_sockets:
                # udp connections should only call stopListening
                self.transport.stopListening()
            else:
                self.transport.loseConnection()
        else:
            if self.connector:
                self.connector.disconnect()

    def _cleanup(self):

        if self.buffer:
            self.buffer.consumer = None
            del self.buffer

        self.handler = None

        del self.transport

        if self.callback_connection:
            if self.callback_connection.can_timeout:
                self.callback_connection.setTimeout(None)
            self.callback_connection.connection = None
            del self.callback_connection

# hint: not actually a buffer
class PassBuffer(object):

    def __init__(self, consumer, callback_onflushed, old_buffer):
        self.consumer = consumer
        self.callback_onflushed = callback_onflushed
        self._is_flushed = False
        self.consumer.addBufferCallback(self._flushed, "buffer empty")
        # swallow the data written to the old buffer
        if old_buffer._buffer_list:
            self.consumer.writeSequence(old_buffer._buffer_list)
            old_buffer._buffer_list[:] = []

    def add(self, b):
        self._is_flushed = False
        self.consumer.write(b)

    def stopWriting(self):
        pass
    
    def is_flushed(self):
        return self._is_flushed
    
    def _flushed(self):
        self._is_flushed = True
        self.callback_onflushed()

        
class OutputBuffer(object):

    # This is an IPullProducer which has an unlimited buffer size,
    # and calls a callback when the buffer is completely flushed

    def __init__(self, callback_onflushed):
        self.consumer = None
        self.registered = False
        self.callback_onflushed = callback_onflushed
        self._buffer_list = []

    def is_flushed(self):
        return (len(self._buffer_list) == 0)

    def attachConsumer(self, consumer):
        self.consumer = consumer

    def add(self, b):
        self._buffer_list.append(b)

        if self.consumer and not self.registered:
            self.beginWriting()

    def beginWriting(self):
        self.stopWriting()
        assert self.consumer, "You must attachConsumer before you beginWriting"
        self.registered = True
        self.consumer.registerProducer(self, False)

    def stopWriting(self):
        if not self.registered:
            return

        self.registered = False
        if self.consumer:
            try:
                self.consumer.unregisterProducer()
            except KeyError:
                # bug in iocpreactor: http://twistedmatrix.com/trac/ticket/1657
                pass

    def resumeProducing(self):
        if not self.registered:
            return
        assert self.consumer, "You must attachConsumer before you resumeProducing"

        to_write = len(self._buffer_list)
        if to_write > 0:
            self.consumer.writeSequence(self._buffer_list)
            del self._buffer_list[:]
            self.callback_onflushed()
        else:
            self.stopWriting()

    def pauseProducing(self):
        pass

    def stopProducing(self):
        pass



class CallbackConnection(object):

    def attachTransport(self, transport, s):
        s.attach_transport(self, transport=transport, reset_timeout=self.optionalResetTimeout)
        self.connection = s

    def connectionMade(self):
        s = self.connection
        s.handler.connection_made(s)
        self.optionalResetTimeout()

        self.factory.rawserver.connectionMade(s)

    def connectionLost(self, reason):
        reactor.callLater(0, self.post_connectionLost, reason)

    # twisted api inconsistancy workaround
    # sometimes connectionLost is called (not queued) from inside write()
    def post_connectionLost(self, reason):
        s = self.connection
        #print s.ip, s.port, reason.getErrorMessage()
        self.factory.rawserver._remove_socket(self.connection, was_connected=True)

    def dataReceived(self, data):
        self.optionalResetTimeout()

        s = self.connection
        s.rawserver._make_wrapped_call(s.handler.data_came_in,
                                       s, data, wrapper=s)

    def datagramReceived(self, data, (host, port)):
        s = self.connection
        s.rawserver._make_wrapped_call(s.handler.data_came_in,
                                       (host, port), data, wrapper=s)

    def optionalResetTimeout(self):
        if self.can_timeout:
            self.resetTimeout()


class CallbackProtocol(CallbackConnection, TimeoutMixin, Protocol):

    def makeConnection(self, transport):
        self.can_timeout = True
        self.setTimeout(self.factory.rawserver.config['socket_timeout'])

        outgoing = False

        key = None
        host = transport.getHost()
        if isinstance(host, address.UNIXAddress):
            key = host.name
        else:
            key = host.port

        try:
            addr = (transport.getPeer().host, transport.getPeer().port)
            c = self.factory.pop_connection_data(addr)
            outgoing = True
        except KeyError:
            # happens when this was an incoming connection
            pass
        except AttributeError:
            # happens when the peer is a unix socket
            pass

        if not outgoing:
            args = self.factory.get_connection_data(key)
            c = ConnectionWrapper(*args)

        self.attachTransport(transport, c)
        Protocol.makeConnection(self, transport)


class CallbackDatagramProtocol(CallbackConnection, DatagramProtocol):

    def startProtocol(self):
        self.can_timeout = False
        self.attachTransport(self.transport, self.connection)
        DatagramProtocol.startProtocol(self)


class ConnectionFactory(ClientFactory):

    def __init__(self):
        self.connection_data = DictWithLists()

    def add_connection_data(self, key, data):
        self.connection_data.push_to_row(key, data)

    def get_connection_data(self, key):
        return self.connection_data.get_from_row(key)

    def pop_connection_data(self, key):
        return self.connection_data.pop_from_row(key)

    def clientConnectionFailed(self, connector, reason):
        peer = connector.getDestination()
        addr = (peer.host, peer.port)
        s = self.pop_connection_data(addr)

        # opt-out
        if not s.dying:
            # this might not work - reason is not an exception
            s.handler.connection_failed(addr, reason)
            s.dying = True

        self.rawserver._remove_socket(s)


# storage for socket creation requestions, and proxy once the connection is made
class SocketRequestProxy(object):
    def __init__(self, port, bind, tos, protocol):
        self.port = port
        self.bind = bind
        self.tos = tos
        self.protocol = protocol
        self.connection = None

    def __getattr__(self, name):
        try:
            return getattr(self.connection, name)
        except:
            raise AttributeError, name

    def close(self):
        # closing the proxy doesn't mean anything.
        # you can stop_listening(), and then start again.
        # the socket only exists while it is listening
        if self.connection:
            self.connection.close()

class RawServerMixin(object):

    def __init__(self, config=None, noisy=True, tos=0):
        self.noisy = noisy
        self.config = config
        if not self.config:
            self.config = {}
        self.tos = tos
        self.sigint_flag = None

    # going away soon. call _context_wrap on the context.
    def _make_wrapped_call(self, _f, *args, **kwargs):
        wrapper = kwargs.pop('wrapper', None)
        try:
            _f(*args, **kwargs)
        except KeyboardInterrupt:
            raise
        except Exception, e:         # hopefully nothing raises strings
            # Incoming sockets can be assigned to a particular torrent during
            # a data_came_in call, and it's possible (though not likely) that
            # there could be a torrent-specific exception during the same call.
            # Therefore read the context after the call.
            context = None
            if wrapper is not None:
                context = wrapper.context
            if context is not None:
                context.got_exception(*sys.exc_info())
            elif self.noisy:
                rawserver_logger.exception("Error in _make_wrapped_call for %s",
                                           _f.__name__)

   
    # must be called from the main thread
    def install_sigint_handler(self, flag = None):
        if flag is not None:
            self.sigint_flag = flag
        signal.signal(signal.SIGINT, self._handler)

    def _handler(self, signum, frame):
        if self.sigint_flag:
            self.external_add_task(0, self.sigint_flag.set)
        elif self.doneflag:
            self.external_add_task(0, self.doneflag.set)
       
        # Allow pressing ctrl-c multiple times to raise KeyboardInterrupt,
        # in case the program is in an infinite loop
        signal.signal(signal.SIGINT, signal.default_int_handler)


class RawServer(RawServerMixin):
    """RawServer encapsulates I/O and task scheduling.

       I/O corresponds to the arrival data on a file descriptor,
       and a task is a scheduled callback.  A task is scheduled
       using add_task or external_add_task.  add_task is used from within the
       thread running the RawServer, external_add_task from other threads.

       tracker.py provides a simple example of how to use RawServer.

        1. creates an instance of RawServer

            r = RawServer(config)

        2. creates a socket by a call to create_serversocket.

            s = r.create_serversocket(config['port'], config['bind'], True)

        3. tells the raw server to listen to the socket and associate
           a protocol handler with the socket.

            r.start_listening(s,
                HTTPHandler(t.get, config['min_time_between_log_flushes']))

        4. tells the raw_server to listen for I/O or scheduled tasks
           until the done_flag is set.

            done_flag = DeferredEvent()
            r.listen_forever(done_flag)

           When a remote client opens a connection, a new socket is
           returned from the server socket's accept method and the
           socket is assigned the same handler as was assigned to the
           server socket.

           As data arrives on a socket, the handler's data_came_in
           member function is called.  It is up to the handler to
           interpret the data and/or pass it on to other objects.
           In the tracker, the HTTP protocol handler passes the arriving data
           to an HTTPConnection object which maintains state specific
           to a given connection.

         For outgoing connections, the call start_connection() is used.
         """


    def __init__(self, config=None, noisy=True, tos=0):
        """config is a dict that contains option-value pairs.

           'tos' is passed to setsockopt to set the IP Type of Service (i.e.,
           DiffServ byte), in IPv4 headers."""
        RawServerMixin.__init__(self, config, noisy, tos)

        self.doneflag = None

        # init is fine until the loop starts
        self.ident = thread.get_ident()
        self.associated = False

        self.single_sockets = set()
        self.udp_sockets = set()
        self.listened = False
        self.connections = 0

        ##############################################################
        if profile:
            try:
                os.unlink(prof_file_name)
            except:
                pass
            self.prof = hotshot.Profile(prof_file_name)

            callLater = reactor.callLater
            def _profile_call(delay, _f, *a, **kw):
                return callLater(delay, self.prof.runcall, _f, *a, **kw)
            reactor.callLater = _profile_call
            o = reactor.callFromThread
            def _profile_call2(_f, *a, **kw):
                return o(self.prof.runcall, _f, *a, **kw)
            reactor.callFromThread = _profile_call2
        ##############################################################

        connectionRateLimitReactor(reactor, self.config.get('max_incomplete', 10))
        # bleh
        self.add_pending_connection = reactor.add_pending_connection
        self.remove_pending_connection = reactor.remove_pending_connection
        self.reactor = reactor

        self.factory = ConnectionFactory()
        self.factory.rawserver = self

        self.factory.protocol = CallbackProtocol

        #l2 = task.LoopingCall(self._print_connection_count)
        #l2.start(1)


    ##############################################################
    def _print_connection_count(self):
        def _sl(x):
            if hasattr(x, "__len__"):
                return str(len(x))
            else:
                return str(x)

        c = len(self.single_sockets)
        u = len(self.udp_sockets)
        c -= u
        #s = "Connections(" + str(id(self)) + "): tcp(" + str(c) + ") upd(" + str(u) + ")"
        #rawserver_logger.debug(s)

        d = dict()
        for s in self.single_sockets:
            state = "None"
            if not s.dying and s.transport:
                try:
                    state = s.transport.state
                except:
                    state = "has transport"
            else:
                state = "No transport"
            if state not in d:
                d[state] = 0
            d[state] += 1
        #rawserver_logger.debug(d)
        print d

        sizes = "cc(" + _sl(self.connections)
        sizes += ") ss(" + _sl(self.single_sockets)
        sizes += ") us(" + _sl(self.udp_sockets) + ")"
        #rawserver_logger.debug(sizes)
        print sizes
    ##############################################################
        
    def get_remote_endpoints(self):
        addrs = [(s.ip, s.port) for s in self.single_sockets]
        return addrs

##    def add_task(self, delay, _f, *args, **kwargs):
##        """Schedule the passed function 'func' to be called after
##           'delay' seconds and pass the 'args'.
##
##           This should only be called by RawServer's thread."""
##        #assert thread.get_ident() == self.ident
##        return reactor.callLater(delay, _f, *args, **kwargs)
    add_task = reactor.callLater

    def external_add_task(self, delay, _f, *args, **kwargs):
        """Schedule the passed function 'func' to be called after
           'delay' seconds and pass 'args'.

           This should be called by threads other than RawServer's thread."""
        if delay == 0:
            return reactor.callFromThread(_f, *args, **kwargs)
        else:
            return reactor.callFromThread(reactor.callLater, delay,
                                          _f, *args, **kwargs)

    def create_unixserversocket(self, filename):
        s = SocketRequestProxy(0, filename, 0, 'unix')

        s.listening_port = reactor.listenUNIX(s.bind, self.factory)
        s.listening_port.listening = True

        return s

    def create_serversocket(self, port, bind='', tos=0):
        s = SocketRequestProxy(port, bind, tos, 'tcp')

        try:
            s.listening_port = reactor.listenTCP(s.port, self.factory,
                                                 interface=s.bind)
        except error.CannotListenError, e:
            if e[0] != 0:
                raise e.socketError
            else:
                raise
        s.listening_port.listening = True

        return s

    def _create_udpsocket(self, port, bind, tos, create_func):
        s = SocketRequestProxy(port, bind, tos, 'udp')

        protocol = CallbackDatagramProtocol()

        c = ConnectionWrapper(None, None, None, tos)
        s.connection = c
        protocol.connection = c

        try:
            s.listening_port = create_func(s.port, protocol, interface=s.bind)
        except error.CannotListenError, e:
            raise e.socketError
        s.listening_port.listening = True

        return s

    def create_udpsocket(self, port, bind='', tos=0):
        return self._create_udpsocket(port, bind, tos,
                                      create_func = reactor.listenUDP)

    def create_multicastsocket(self, port, bind='', tos=0):
        return self._create_udpsocket(port, bind, tos,
                                      create_func = reactor.listenMulticast)

    def _start_listening(self, s):
        if not s.listening_port.listening:
            s.listening_port.startListening()
            s.listening_port.listening = True

    def _get_data_key(self, serversocket):
        if serversocket.protocol == 'tcp':
            key = serversocket.port
        elif serversocket.protocol == 'unix':
            key = serversocket.bind
        else:
            raise TypeError("Unknown protocol: " + str(serversocket.protocol))
        return key        

    def start_listening(self, serversocket, handler, context=None):
        self.factory.add_connection_data(self._get_data_key(serversocket),
                                         (self, handler, context, serversocket.tos))

        self._start_listening(serversocket)

    def start_listening_udp(self, serversocket, handler, context=None):
        c = serversocket.connection
        c.post_init(self, handler, context)

        self._start_listening(serversocket)

        self.udp_sockets.add(c)

    start_listening_multicast = start_listening_udp

    def stop_listening(self, serversocket):
        listening_port = serversocket.listening_port
        try:
            listening_port.stopListening()
        except AttributeError:
            # AttributeError: 'MulticastPort' object has no attribute 'handle_disconnected_stopListening'
            # sigh.
            pass
        listening_port.listening = False
        if serversocket.protocol != 'udp':
            self.factory.pop_connection_data(self._get_data_key(serversocket))

    def stop_listening_udp(self, serversocket):
        self.stop_listening(serversocket)

        self.udp_sockets.remove(serversocket.connection)
        self.single_sockets.remove(serversocket.connection)

    stop_listening_multicast = stop_listening_udp

    def start_connection(self, dns, handler, context=None, do_bind=True):
        addr = dns[0]
        port = int(dns[1])

        if len(letters.intersection(addr)) > 0:
            rawserver_logger.warning("Don't pass host names to RawServer")
            # this blocks, that's why we throw the warning
            addr = socket.gethostbyname(addr)

        bindaddr = None
        if do_bind:
            bindaddr = self.config['bind']
            if isinstance(bindaddr, str) and len(bindaddr) >= 0:
                bindaddr = (bindaddr, 0)
            else:
                bindaddr = None

        c = ConnectionWrapper(self, handler, context, self.tos)

        self.factory.add_connection_data((addr, port), c)

        connector = reactor.connectTCP(addr, port, self.factory, bindAddress=bindaddr)
        c.connector = connector

        self.single_sockets.add(c)
        return c

    def associate_thread(self):
        assert not self.associated, \
               "RawServer has already been associated with a thread"
        self.ident = thread.get_ident()
        self.associated = True

    def listen_forever(self, doneflag=None):
        """Main event processing loop for RawServer.
           RawServer listens until the doneFlag is set by some other
           thread.  The doneFlag tells all threads to clean-up and then
           exit."""
        if not doneflag:
            doneflag = DeferredEvent()
        assert isinstance(doneflag, DeferredEvent)
        self.doneflag = doneflag
        if not self.associated:
            self.associate_thread()
        if self.listened:
            Exception(_("listen_forever() should only be called once per reactor."))
        reactor.callLater(0, self.doneflag.addCallback, self._safestop)
        self.listened = True

        reactor.suggestThreadPoolSize(1)
        if noSignals:
            reactor.run(installSignalHandlers=False)
        else:
            reactor.run()

        if profile:
            self.prof.close()
            stats = hotshot.stats.load(prof_file_name)
            stats.strip_dirs()
            stats.sort_stats('time', 'calls')
            print "Rawserver MainLoop Profile:"
            stats.print_stats(20)

    def listen_once(self, period=1e9):
        rawserver_logger.warning(_("listen_once() might not return until there is activity, and might not process the event you want. Use listen_forever()."))
        reactor.iterate(period)

    def stop(self):
        self.doneflag.set()

    def _safestop(self, r=None):
        if not threadable.isInIOThread():
            self.external_add_task(0, self._stop)
        else:
            self._stop()

    def _stop(self, r=None):
        assert thread.get_ident() == self.ident

        connections = list(self.single_sockets)
        for connection in connections:
            try:
                connection.close()
            except:
                pass

        reactor.suggestThreadPoolSize(0)
        try:
            reactor.stop()
        except RuntimeError:
            # exceptions.RuntimeError: can't stop reactor that isn't running
            pass

    def _remove_socket(self, s, was_connected=False):
        # opt-out
        if not s.dying:
            self._make_wrapped_call(s.handler.connection_lost, s, wrapper=s)

        s._cleanup()

        self.single_sockets.remove(s)

        if was_connected:
            self.connections -= 1
            
    def connectionMade(self, s):
        self.connections += 1
        



# by Greg Hazel

import random
from twisted.internet import task
from BitTorrent.obsoletepythonsupport import set

class InternetSubscriber(object):

    def internet_active(self):
        pass

    def try_one_connection(self):
        pass

class InternetWatcher(object):
    def __init__(self, rawserver):

        self.rawserver = rawserver
        old_connectionMade = rawserver.connectionMade
        def connectionMade(s):
            if rawserver.connections == 0:
                self._first_connection()
            old_connectionMade(s)
        assert not hasattr(rawserver, "internet_watcher"), \
                "rawserver already has conncetion rate limiter installed"
        rawserver.connectionMade = connectionMade
        rawserver.internet_watcher = self

        self.subscribers = set()
        self.internet_watcher = task.LoopingCall(self._internet_watch)
        self.internet_watcher.start(5)

    def add_subscriber(self, s):
        self.subscribers.add(s)

    def remove_subscriber(self, s):
        self.subscribers.remove(s)

    def _internet_watch(self):
        if self.rawserver.connections != 0 or not self.subscribers:
            return
        l = list(self.subscribers)
        random.shuffle(l)
        for s in l:
            if s.try_one_connection():
                break

    def _first_connection(self):
        for s in self.subscribers:
            s.internet_active()



import sys
sys.path = ['.',] + sys.path #HACK

import os
from BitTorrent.platform import plugin_path, app_root
plugin_path.append(os.path.join(app_root[:-5], 'BitTorrent', 'Plugins')) #HACK

from BitTorrent.FeedManager import FeedManager

def gui_wrap(f, *a):
    f(*a)

feedmanager = FeedManager({}, gui_wrap)

# Test RSS 2 feed:
feed = 'http://www.prodigem.com/torrents/rss/pep_delicious.xml'
feedmanager.new_channel(feed)

# Test RAW feed:
feed = 'http://search.bittorrent.com/search.jsp?query=Ubuntu&Submit2=Search'
feedmanager.new_channel(feed)

import time
time.sleep(10)



# A very very minimal BeautifulSoup immitation.
#
# BS uses SGMLlib to parse, which converts everything to lower case.
# This uses real xml parsing to mimic the parts of BS we use.

import xml.dom.minidom

def _getText(node):
    nodelist = node.childNodes
    rc = []
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc.append(str(node.data))
    return rc

def _getNodesAsTags(root):
    nodelist = root.childNodes
    tags = []
    for node in nodelist:
        if node.nodeType == node.ELEMENT_NODE:
            tags.append(Tag(node))
    return tags

class Tag(object):
    def __init__(self, node):
        self.node = node
        self.name = node.nodeName
        self.contents = _getNodesAsTags(self.node)
        text = _getText(self.node)
        self.contents += text
        self.text = ''.join(text)

    def child_elements(self):
        children = []
        for tag in self.contents:
            if isinstance(tag, Tag):
                children.append(tag)
        return children

    def get(self, tagname):
        got = self.first(tagname)
        if got:
            return got.text

    def first(self, tagname):
        found = None
        
        for tag in self.contents:
            if isinstance(tag, Tag):
                if tag.name == tagname:
                    found = tag
                    break
        
        return found
   
class BeautifulSupe(object):

    def __init__(self, data):
        #please don't give us your null terminators
        data = data.strip(chr(0))
        self.dom = xml.dom.minidom.parseString(data)
    
    def first(self, tagname, root = None):
        found = None
        if root == None:
            e = self.dom.getElementsByTagName(tagname)
            if len(e) > 0:
                found = e[0]
        else:
            for node in root.childNodes:
                if node.nodeName == tagname:
                    found = node
                    break

        if not found:
            return None

        tag = Tag(found)
        return tag

    def fetch(self, tagname, restraints = {}):
        e = self.dom.getElementsByTagName(tagname)

        matches = []

        for node in e:
            match = 1
            
            for restraint in restraints:
                f = self.first(restraint, node)
                if not f:
                    match = 0
                    break
                text = restraints[restraint]
                if not f.contents[0].startswith(text):
                    match = 0
                    break
                
            if match:
                tag = Tag(node)
                matches.append(tag)

        return matches


    def scour(self, prefix, suffix = None, node = None):
        if node is None:
            root = self.dom.getElementsByTagName(self.dom.documentElement.tagName)[0]
            node = root

        matches = []

        for node in node.childNodes:
            match = 0
            
            name = node.nodeName

            if name.startswith(prefix):
                if suffix:
                    if name.endswith(suffix):
                        match = 1
                else:
                    match = 1
                    
            if match:
                tag = Tag(node)
                matches.append(tag)

            matches += self.scour(prefix, suffix, node)

        return matches        




# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

import sys
import weakref
import traceback
#import thread
import BitTorrent.stackthreading as threading

debug = False

class Deferred(object):
    __slots__ = ['callbacks', 'errbacks',
                 'calledBack', 'erredBack', 'called_errbacks',
                 'results', 'failures',
                 'weakref', '__weakref__',
                 'stack']
    def __init__(self):
        if debug:
            self.stack = traceback.format_stack()[:-1]
        else:
            self.stack = None
        #self.ident = thread.get_ident()
        self.callbacks = []
        self.errbacks = []
        self.calledBack = False
        self.erredBack = False
        # python is really, really dumb
        self.called_errbacks = [False,]
        self.results = []
        self.failures = []
        def pseudo_del(ref, failures=self.failures,
                       called_errbacks=self.called_errbacks,
                       stack=self.stack):
            # iterpreter shutdown
            if sys is None:
                return
            # so this checks to see if -any- errback has been called, not -all-.
            # See twisted deferreds for a better (although still broken!)
            # implementation of the correct metric.
            if failures and not called_errbacks[0]:
                sys.stderr.write("Unhandled error in BT Deferred:\n")
                if stack:
                    sys.stderr.writelines(stack)
                for failure in failures:
                    try:
                        traceback.print_exception(*failure)
                    except Exception, e:
                        sys.stderr.write("%s\n" % str(failure))
        self.weakref = weakref.ref(self, pseudo_del)

    def getResult(self):
        #if self.ident != thread.get_ident():
        #    sys.stderr.write("getResult from the wrong thread!\n")
        #    sys.stderr.writelines(self.stack)
        #assert self.ident == thread.get_ident()
        self.called_errbacks[0] = True
        if self.failures:
            # what should I do with multiple failures?
            assert isinstance(self.failures[0], tuple), "Not a known failure type:" + str(self.failures[0])
            exc_type, value, tb = self.failures[0]
            try:
                if tb.__class__.__name__ == 'FakeTb':
                    tb = tb.tb_orig
            except:
                pass
            raise exc_type, value, tb
        if len(self.results) > 1:
            return self.results
        elif len(self.results) == 1:
            return self.results[0]
        return None

    def addCallback(self, cb, *args, **kwargs):
        assert callable(cb)
        self.callbacks.append((cb, args, kwargs))
        if self.calledBack:
            self.doCallbacks(self.results, [(cb, args, kwargs)])
        return self

    def addErrback(self, cb, *args, **kwargs):
        assert callable(cb)
        self.errbacks.append((cb, args, kwargs))
        if self.erredBack:
            self.doCallbacks(self.failures, [(cb, args, kwargs)])
        return self

    def addCallbacks(self, cb, eb, args=(), kwargs={},
                     ebargs=(), ebkwargs={}):
        self.addCallback(cb, *args, **kwargs)
        self.addErrback(eb, *ebargs, **ebkwargs)

    def chainDeferred(self, d):
        return self.addCallbacks(d.callback, d.errback)

    def callback(self, result):
        self.results.append(result)
        self.calledBack = True
        if self.callbacks:
            self.doCallbacks([result], self.callbacks)

    def errback(self, failed):
        self.failures.append(failed)
        self.erredBack = True
        if self.errbacks:
            self.called_errbacks[0] = True
            self.doCallbacks([failed], self.errbacks)

    def doCallbacks(self, results, callbacks):
        #if self.ident != thread.get_ident():
        #    sys.stderr.write("doCallbacks from the wrong thread!\n")
        #    sys.stderr.writelines(self.stack)
        #assert self.ident == thread.get_ident()
        for result in results:
            for cb, args, kwargs in callbacks:
                result = cb(result, *args, **kwargs)


# not totally safe, but a start.
# This lets you call callback/errback from any thread.
# The next step would be for addCallbak and addErrback to be safe.
class ThreadableDeferred(Deferred):
    def __init__(self, queue_func):
        assert callable(queue_func)
        self.queue_func = queue_func
        Deferred.__init__(self)

    def callback(self, result):
        self.queue_func(Deferred.callback, self, result)

    def errback(self, result):
        self.queue_func(Deferred.errback, self, result)
        

# go ahead and forget to call start()!
class ThreadedDeferred(Deferred):

    def __init__(self, queue_func, f, *args, **kwargs):
        Deferred.__init__(self)
        daemon = False
        if 'daemon' in kwargs:
            daemon = kwargs.pop('daemon')
        self.f = f
        start = True
        if queue_func is None:
            start = False
            queue_func = lambda f, *a, **kw : f(*a, **kw)
        self.queue_func = queue_func
        self.args = args
        self.kwargs = kwargs
        self.t = threading.Thread(target=self.run)
        self.t.setDaemon(daemon)
        if start:
            self.start()

    def start(self):
        self.t.start()

    def run(self):
        try:
            r = self.f(*self.args, **self.kwargs)
            self.queue_func(self.callback, r)
        except:
            self.queue_func(self.errback, sys.exc_info())


class DeferredEvent(Deferred, threading._Event):
    def __init__(self, *a, **kw):
        threading._Event.__init__(self)
        Deferred.__init__(self, *a, **kw)
        
    def set(self):
        threading._Event.set(self)
        self.callback(None) # hmm, None?


def run_deferred(df, f, *a, **kw):
    try:
        v = f(*a, **kw)
    except:
        df.errback(sys.exc_info())
    else:
        df.callback(v)
    return df
        



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# written by Matt Chisholm, Steven Hazel, and Greg Hazel

import locale
import traceback
import wx
from UserDict import IterableUserDict
from BitTorrent.obsoletepythonsupport import set
from wx.lib.mixins.listctrl import ColumnSorterMixin
from wx.lib.mixins.listctrl import getListCtrlSelection
import os
if os.name == 'nt':
    LVM_FIRST = 0x1000
    LVM_SETSELECTEDCOLUMN = (LVM_FIRST + 140)
    import win32api


def highlight_color(c):
    if c > 240:
        c *= 0.97
    else:
        c = min(c * 1.10, 255)
    return int(c)


SEL_FOC = wx.LIST_STATE_SELECTED | wx.LIST_STATE_FOCUSED
def selectBeforePopup(ctrl, pos):
    """Ensures the item the mouse is pointing at is selected before a popup.

    Works with both single-select and multi-select lists."""

    if not isinstance(ctrl, wx.ListCtrl):
        return

    n, flags = ctrl.HitTest(pos)
    if n < 0:
        return

    if not ctrl.GetItemState(n, wx.LIST_STATE_SELECTED):
        for i in xrange(ctrl.GetItemCount()):
            ctrl.SetItemState(i, 0, SEL_FOC)
        ctrl.SetItemState(n, SEL_FOC, SEL_FOC)

class ContextMenuMixin(object):
    def __init__(self):
        self.context_menu = None
        self.column_context_menu = None
        self.Bind(wx.EVT_CONTEXT_MENU, self.OnContextMenu)
        self.Bind(wx.EVT_LIST_COL_RIGHT_CLICK, self.OnColumnContextMenu)

    def SetContextMenu(self, menu):
        self.context_menu = menu

    def SetColumnContextMenu(self, menu):
        self.column_context_menu = menu

    def OnColumnContextMenu(self, event):
        if self.column_context_menu:
            self.PopupMenu(self.column_context_menu)

    def OnContextMenu(self, event):
        pos = self.ScreenToClient(event.GetPosition())
        top = self.GetItemRect(self.GetTopItem())
        if pos[1] < top.y:
            event.Skip()
            return

        pos -= self._get_origin_offset()
        self.DoPopup(pos)

    def DoPopup(self, pos):
        """ pos should be in client coords """
        if self.context_menu:
            selectBeforePopup(self, pos)
            selection = getListCtrlSelection(self)
            if len(selection) > 0:
                self.PopupMenu(self.context_menu)
                return


class BTListColumn(wx.ListItem):

    def __init__(self, text, sample_data, renderer=None, comparator=None, enabled=True, width=50):
        wx.ListItem.__init__(self)
        self.SetText(text)
        self.renderer = renderer
        self.comparator = comparator
        self.enabled = enabled
        self.sample_data = sample_data
        self.width = width


class BTListRow(IterableUserDict):
    __slots__ = ['data', 'index']

    def __init__(self, index, data):
        self.data = data
        self.index = index

    def __getitem__(self, i):
        return self.data[i]


class BTListCtrl(wx.ListCtrl, ColumnSorterMixin, ContextMenuMixin):
    # Part of this class based on:
    # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/426407
    icon_size = 16

    def __init__(self, parent):
        wx.ListCtrl.__init__(self, parent, wx.ID_ANY, style=wx.LC_REPORT)
        ContextMenuMixin.__init__(self)

        self.il = wx.ImageList(self.icon_size, self.icon_size)
        # TODO: use a real icon
        self.il.Add(self.draw_blank())
        self.il.Add(self.draw_sort_arrow('up'))
        self.il.Add(self.draw_sort_arrow('down'))
        self.SetImageList(self.il, wx.IMAGE_LIST_SMALL)

        self.update_enabled_columns()

        for i, name in enumerate(self.enabled_columns):
            column = self.columns[name]
            column.SetColumn(i)
            self.InsertColumnItem(i, column)

        self.itemData_to_row = {}
        self.index_to_itemData = {}

        self.selected_column = None
        self.SelectColumn(self.enabled_columns[0])

        cmenu = wx.Menu()
        for name in self.column_order:
            column = self.columns[name]
            id = wx.NewId()
            cmenu.AppendCheckItem(id, column.GetText())
            cmenu.Check(id, column.enabled)
            self.Bind(wx.EVT_MENU,
                      lambda e, c=column, id=id: self.toggle_column(c, id, e),
                      id=id)
        self.SetColumnContextMenu(cmenu)

        ColumnSorterMixin.__init__(self, len(self.enabled_columns))
        self._last_scrollpos = 0
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)

        self.default_rect = wx.Rect(0,0)

    def OnEraseBackground(self, event=None):
        nsp = self.GetScrollPos(wx.VERTICAL)
        if self._last_scrollpos != nsp:
            self._last_scrollpos = nsp
            # should only refresh visible items, hmm
            wx.CallAfter(self.Refresh)
        dc = wx.ClientDC(self)
        # erase the section of the background which is not covered by the
        # items or the selected column highlighting
        dc.SetBackground(wx.Brush(self.GetBackgroundColour()))
        f = self.GetRect()
        r = wx.Region(0, 0, f.width, f.height)
        x = self.GetVisibleViewRect()
        offset = self._get_origin_offset(include_header=True)
        x.Offset(offset)
        r.SubtractRect(x)
        if '__WXMSW__' in wx.PlatformInfo:
            c = self.GetColumnRect(self.enabled_columns.index(self.selected_column))
            r.SubtractRect(c)
        dc.SetClippingRegionAsRegion(r)
        dc.Clear()

        if '__WXMSW__' in wx.PlatformInfo:
            # draw the selected column highlighting under the items
            dc.DestroyClippingRegion()
            r = wx.Region(0, 0, f.width, f.height)
            r.SubtractRect(x)
            dc.SetClippingRegionAsRegion(r)
            dc.SetPen(wx.TRANSPARENT_PEN)
            hc = wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW)
            r = highlight_color(hc.Red())
            g = highlight_color(hc.Green())
            b = highlight_color(hc.Blue())
            hc.Set(r, g, b)
            dc.SetBrush(wx.Brush(hc))
            dc.DrawRectangle(c.x, c.y, c.width, c.height)

    def update_enabled_columns(self):
        self.enabled_columns = [name for name in self.column_order
                                if self.columns[name].enabled]

    def toggle_column(self, tcolumn, id, event):
        self.update_column_widths()
        sort_col = self.get_sort_column()

        tcolumn.enabled = not tcolumn.enabled
        self.column_context_menu.Check(id, tcolumn.enabled)

        self.update_enabled_columns()

        if not tcolumn.enabled:
            self.DeleteColumn(tcolumn.GetColumn())

        new_col_names = []
        for i, name in enumerate(self.enabled_columns):
            column = self.columns[name]
            column.SetColumn(i)
            if column == tcolumn:
                self.InsertColumnItem(i, column)
                new_col_names.append(name)
            self.SetColumnWidth(column.GetColumn(), column.width)

        self.SetColumnCount(len(self.enabled_columns))
        self.SortListItems(col=sort_col)

        for itemData in self.itemData_to_row.iterkeys():
            self.InsertRow(itemData, self.itemData_to_row[itemData],
                           sort=True, force_update_columns=new_col_names)
        #self.SortItems()

    def set_default_widths(self):
        # must be called before *any* data is put into the control.
        sample_data = {}
        for name in self.column_order:
            sample_data[name] = self.columns[name].sample_data
        sample_row = BTListRow(None, sample_data)

        self.InsertRow(-1, sample_row)
        for name in self.column_order:
            column = self.columns[name]
            if name in self.enabled_columns:
                self.SetColumnWidth(column.GetColumn(), wx.LIST_AUTOSIZE)
                column.width = self.GetColumnWidth(column.GetColumn())
            dc = wx.ClientDC(self)
            header_width = dc.GetTextExtent(column.GetText())[0]
            header_width += 4 # arbitrary allowance for header decorations
            column.width = max(column.width, header_width)
            if name in self.enabled_columns:
                self.SetColumnWidth(column.GetColumn(), column.width)
        self.default_rect = self.GetItemRect(0)
        self.DeleteRow(-1)

    def _get_origin_offset(self, include_header=None):

        if include_header is None:
            # Hm, I think this is a legit bug in wxGTK
            if '__WXGTK__' in wx.PlatformInfo:
                include_header = True
            else:
                include_header = False

        if include_header:            
            i = self.GetTopItem()
            try:
                r = self.GetItemRect(i)
            except wx._core.PyAssertionError:
                r = self.default_rect
            return (r.x, r.y)
        return (0, 0)

    def add_image(self, image):
        b = wx.BitmapFromImage(image)
        assert b.Ok(), "The image (%s) is not valid." % str(image)
        if '__WXMSW__' in wx.PlatformInfo:
            # hack for 16-bit color mode
            if b.GetDepth() > 24: # I mean, has alpha
                b.SetMask(wx.Mask(b, wx.Colour(0, 0, 0)))
        return self.il.Add(b)

    # Arrow drawing
    def draw_blank(self):
        b = wx.EmptyBitmap(self.icon_size, self.icon_size)
        dc = wx.MemoryDC()
        dc.SelectObject(b)
        dc.SetBackgroundMode(wx.TRANSPARENT)
        dc.Clear()
        dc.SelectObject(wx.NullBitmap)
        b.SetMask(wx.Mask(b, (255, 255, 255)))
        return b

    # this builds an identical arrow to the windows listctrl arrows, in themed
    # and non-themed mode.
    def draw_sort_arrow(self, direction):
        b = wx.EmptyBitmap(self.icon_size, self.icon_size)
        w, h = b.GetSize()
        ho = (h - 5) / 2
        dc = wx.MemoryDC()
        dc.SelectObject(b)
        colour = wx.SystemSettings_GetColour(wx.SYS_COLOUR_GRAYTEXT)
        dc.SetBackgroundMode(wx.TRANSPARENT)
        dc.Clear()
        dc.SetPen(wx.Pen(colour))
        for i in xrange(5):
            if direction == 'down':
                j = 4 - i
            else:
                j = i
            dc.DrawLine(i,j+ho,9-i,j+ho)
        dc.SelectObject(wx.NullBitmap)
        b.SetMask(wx.Mask(b, (255, 255, 255)))
        return b

    def GetBottomItem(self):
        total = self.GetItemCount()
        top = self.GetTopItem()
        pp = self.GetCountPerPage()
        # I purposefully do not subtract 1 from pp, because pp is whole items
        bottom = min(top + pp, total - 1)
        return bottom

    def SelectColumn(self, col):
        """Color the selected column (MSW only)"""
        if self.selected_column == col:
            return
        col_num = self.enabled_columns.index(col)
        if os.name == 'nt':
            win32api.PostMessage(self.GetHandle(),
                                 LVM_SETSELECTEDCOLUMN, col_num, 0)

            if self.selected_column is not None:
                self.RefreshCol(self.selected_column)
            self.RefreshCol(col)
            self.selected_column = col

    def render_column_text(self, row, name):
        """Renders the column value into a string"""
        item = self.columns[name]
        value = row[name]
        if value is None:
            text = '?'
        elif item.renderer is not None:
            try:
                text = item.renderer(value)
            except:
                text = '?'
                # BUG: for debugging only
                traceback.print_exc()
        else:
            text = unicode(value)
        return text


    def get_column_image(self, row):
        return None

    def _update_indexes(self, start = 0):
        for i in xrange(start, self.GetItemCount()):
            itemData = self.GetItemData(i)
            self.itemData_to_row[itemData].index = i
            self.index_to_itemData[i] = itemData


    # Used by the ColumnSorterMixin, see wx/lib/mixins/listctrl.py
    def SortItems(self, sorter=None):

        if sorter is None:
            sorter = self.GetColumnSorter()

        # TODO:
        # this step is to see if the list needs resorted.
        # improve this by stopping the first time the order would be changed.
        d = [None,] * self.GetItemCount()
        for i in xrange(len(d)):
            # use real GetItemData, so the sorter can translate
            d[i] = wx.ListCtrl.GetItemData(self, i)
        n = list(d)
        n.sort(sorter)

        if n != d:
            wx.ListCtrl.SortItems(self, sorter)

            self._update_indexes()

        self.SelectColumn(self.enabled_columns[self._col])

    def SortListItems(self, col=-1, ascending=1):
        if col in self.enabled_columns:
            col = self.enabled_columns.index(col)
        else:
            col = 0

        ColumnSorterMixin.SortListItems(self, col=col, ascending=ascending)


    def GetSelection(self):
        return getListCtrlSelection(self)


    def GetSelectionData(self):
        indexes = self.GetSelection()
        data = []
        for i in indexes:
            data.append(self.GetItemData(i))
        return data

    # Used by the ColumnSorterMixin, see wx/lib/mixins/listctrl.py
    def GetListCtrl(self):
        return self

    # Used by the ColumnSorterMixin, see wx/lib/mixins/listctrl.py
    def GetSortImages(self):
        return (1, 2)


    def GetColumnSorter(self):
        """Returns a callable object to be used for comparing column values when sorting."""
        return self.__ColumnSorter

    def TranslateItemData(self, itemData):
        return itemData

    def __ColumnSorter(self, itemData1, itemData2):
        """Allows custom compare functions, in self.colcmps."""
        col = self._col
        ascending = self._colSortFlag[col]

        if col < len(self.enabled_columns):
            name = self.enabled_columns[col]
        else:
            name = self.column_order[0]

        itemData1 = self.TranslateItemData(itemData1)
        itemData2 = self.TranslateItemData(itemData2)
        item1 = self.itemData_to_row[itemData1][name]
        item2 = self.itemData_to_row[itemData2][name]

        column = self.columns[name]

        if column.comparator != None:
            # use custom cmp method
            cmpVal = column.comparator(item1, item2)
        elif isinstance(item1, str) or isinstance(item2, str):
            # Internationalization of string sorting with locale module
            cmpVal = locale.strcoll(unicode(item1), unicode(item2))
        else:
            cmpVal = cmp(item1, item2)

        # If the items are equal then pick something else to make the sort value unique
        if cmpVal == 0:
            cmpVal = apply(cmp, self.GetSecondarySortValues(col, itemData1, itemData2))

        if ascending:
            return cmpVal
        else:
            return -cmpVal

    def RefreshCol(self, col):
        if col in self.enabled_columns:
            self.RefreshRect(self.GetColumnRect(self.enabled_columns.index(col)))

    def HitTestColumn(self, pos):
        """ pos should be in client coords """
        i = self.GetTopItem()
        r = self.GetItemRect(i)
        x, y = self._get_origin_offset()
        if pos[1] >= (r.y - y):
            return None
        loc = 0
        for n in xrange(self.GetColumnCount()):
            loc += self.GetColumnWidth(n)
            if pos[0] < loc:
                return n

    def GetVisibleViewRect(self):
        width = 0
        for n in xrange(self.GetColumnCount()):
            width += self.GetColumnWidth(n)
        height = 0
        if self.GetItemCount() > 0:
            i = self.GetTopItem()
            r1 = self.GetItemRect(i)
            last = min(i + self.GetCountPerPage(), self.GetItemCount() - 1)
            r2 = self.GetItemRect(last)
            height = r2.y + r2.height - r1.y
        x, y = self._get_origin_offset()
        # there is a 2 pixel strip on either side which is not part of the item
        if '__WXMSW__' in wx.PlatformInfo:
            x += 2
            width -= 4
            
        return wx.Rect(x, y, x+width, y+height)
        
    def GetViewRect(self):
        width = 0
        for n in xrange(self.GetColumnCount()):
            width += self.GetColumnWidth(n)
        height = 0
        if self.GetItemCount() > 0:
            r1 = self.GetItemRect(0)
            r2 = self.GetItemRect(self.GetItemCount() - 1)
            height = r2.y + r2.height - r1.y
        x, y = self._get_origin_offset()
        return wx.Rect(x, y, x+width, y+height)

    def _GetColumnWidthExtent(self, col):
        col_locs = [0]
        loc = 0
        num_cols = min(col+1, self.GetColumnCount())
        for n in xrange(num_cols):
            loc += self.GetColumnWidth(n)
            col_locs.append(loc)

        x0 = col_locs[col]
        x1 = col_locs[col+1] - 1
        return x0, x1

    def GetColumnRect(self, col):
        x0, x1 = self._GetColumnWidthExtent(col)

        r = self.GetItemRect(0)
        y0 = r.y
        y1 = self.GetClientSize()[1]

        x_scroll = self.GetScrollPos(wx.HORIZONTAL)

        return wx.RectPP(wx.Point(x0 - x_scroll, y0),
                         wx.Point(x1 - x_scroll, y1))


    def GetCellRect(self, row, col):
        x0, x1 = self._GetColumnWidthExtent(col)

        r = self.GetItemRect(row)
        y0 = r.y
        y1 = r.GetBottom()

        x_scroll = self.GetScrollPos(wx.HORIZONTAL)

        return wx.RectPP(wx.Point(x0 - x_scroll, y0),
                         wx.Point(x1 - x_scroll, y1))

    def DeselectAll(self):
        self.SetItemState(-1, 0, wx.LIST_STATE_SELECTED)
        # fallback. for extremely long lists a generator should be used
        #for i in xrange(self.GetItemCount()):
        #    self.SetItemState(i, 0, wx.LIST_STATE_SELECTED)

    def InsertRow(self, itemData, lr, sort=True, colour=None,
                  force_update_columns=[]):

        # pre-render all data
        image_id = self.get_column_image(lr)
        row_text = {}
        for i, name in enumerate(self.enabled_columns):
            row_text[i] = self.render_column_text(lr, name)

        if itemData not in self.itemData_to_row:
            # this is Add
            # TODO: insert in sorted order instead of sorting
            i = self.InsertStringItem(self.GetItemCount(), '')
            lr.index = i
            self.SetItemData(i, itemData)
            for col in xrange(len(self.enabled_columns)):
                self.SetStringItem(index=lr.index, col=col,
                                   label=row_text[col])
        else:
            # this is Update
            old_lr = self.itemData_to_row[itemData]
            lr.index = old_lr.index
            for col, colname in enumerate(self.enabled_columns):
                if lr[colname] != old_lr[colname] or \
                       colname in force_update_columns:
                    self.SetStringItem(index=lr.index, col=col,
                                       label=row_text[col])
        self.itemData_to_row[itemData] = lr
        self.index_to_itemData[i] = itemData

        if colour is not None:
            if self.GetItemTextColour(lr.index) != colour:
                self.SetItemTextColour(lr.index, colour)

        self.SetItemImage(lr.index, image_id)

        if sort:
            # TODO: move to update-only once things are inserted in sorted order
            self.SortItems()

    def SetItemImage(self, index, image_id):
        item = self.GetItem(index)
        if item.GetImage() != image_id:
            wx.ListCtrl.SetItemImage(self, index, image_id)

    def DeleteRow(self, itemData):
        lr = self.itemData_to_row.pop(itemData)
        self.DeleteItem(lr.index)
        self._update_indexes(lr.index)

    def GetRow(self, index):
        itemData = self.index_to_itemData[index]
        return self.itemData_to_row[itemData]

    # Persistence methods
    def get_column_widths(self):
        widths = {}
        for name in self.column_order:
            column = self.columns[name]
            if column.enabled:
                column.width = self.GetColumnWidth(column.GetColumn())
            widths[name] = column.width
        return widths

    def set_column_widths(self, widths):
        # backward compatibility with development versions
        if isinstance(widths, list):
            return

        for name, width in widths.iteritems():
            column = self.columns[name]
            column.width = width
            if column.enabled:
                self.SetColumnWidth(column.GetColumn(), column.width)

    def update_column_widths(self):
        for name in self.enabled_columns:
            column = self.columns[name]
            column.width = self.GetColumnWidth(column.GetColumn())

    def get_sort_column(self):
        if self._col < len(self.enabled_columns):
            sort_col = self.enabled_columns[self._col]
        else:
            sort_col = None

        return sort_col

    def get_sort_order(self):
        return self._colSortFlag[self._col]


class HashableListView(BTListCtrl):
    """wx.ListCtrl expects integer identifiers for each row.  This
    subclass lets you use any hashable as the identifier instead."""

    def __init__(self, *a, **k):
        BTListCtrl.__init__(self, *a, **k)
        self.itemData_to_hashable = {}
        self.hashable_to_itemData = {}
        self.unique_index = 0

    def GetNewItemData(self):
        self.unique_index += 1
        return self.unique_index

    def GetItemData(self, index):
        itemData = BTListCtrl.GetItemData(self, index)
        return self.itemData_to_hashable[itemData]

    def SetItemData(self, index, hashable):
        itemData = self.hashable_to_itemData[hashable]
        BTListCtrl.SetItemData(self, index, itemData)

    def InsertRow(self, hashable, row, sort=True, colour=None,
                  force_update_columns=[]):
        if hashable not in self.hashable_to_itemData:
            itemData = self.GetNewItemData()
            self.hashable_to_itemData[hashable] = itemData
            self.itemData_to_hashable[itemData] = hashable
        b = BTListCtrl.InsertRow(self, hashable, row, sort=sort,
                                 colour=colour,
                                 force_update_columns=force_update_columns)
        return b

    def DeleteRow(self, hashable):
        itemData = self.hashable_to_itemData.pop(hashable)
        del self.itemData_to_hashable[itemData]
        return BTListCtrl.DeleteRow(self, hashable)

    def TranslateItemData(self, itemData):
        return self.itemData_to_hashable[itemData]

    def GetRowFromKey(self, hashable):
        return self.itemData_to_row[hashable]



# -*- coding: UTF-8 -*-
# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.


app_name = 'BitTorrent'
version = '4.20.4'

URL = 'http://www.bittorrent.com/'
DONATE_URL = URL + 'donate.html?client=%(client)s'
FAQ_URL = URL + 'FAQ.html?client=%(client)s'
SEARCH_URL = 'http://www.bittorrent.com/search_result.html?client=%(client)s&search=%(search)s'
#LOCALE_URL = URL + 'translations/'
LOCALE_URL = 'http://translations.bittorrent.com/'

NAG_FREQUENCY = 3
PORT_RANGE = 5

import sys
assert sys.version_info >= (2, 3, 0), _("Python %s or newer required") % '2.3.0'
import os
import time
import shutil
import urllib
import codecs
import logging
import logging.handlers
from StringIO import StringIO

from BitTorrent import atexit_threads

class BTFailure(Exception):
    pass

# failure due to user error.  Should output differently (e.g., not outputting
# a backtrace).
class UserFailure(BTFailure):
    pass

class InfoHashType(str):
    def __repr__(self):
        return self.encode('hex')
    def short(self):
        return repr(self)[:8]


branch = None
p = os.path.realpath(os.path.split(sys.argv[0])[0])
if os.path.exists(os.path.join(p, '.cdv')):
    branch = os.path.split(p)[1]
del p


def urlquote_error(error):
     s = error.object[error.start:error.end]
     s = s.encode('utf8')
     s = urllib.quote(s)
     s = s.decode('ascii')
     return (s, s.end)

codecs.register_error('urlquote', urlquote_error)


from BitTorrent.language import languages, language_names
from BitTorrent.platform import get_temp_subdir, get_home_dir, get_dot_dir, is_frozen_exe


if os.name == 'posix':
    if os.uname()[0] == "Darwin":
        from BitTorrent.translation import _

if "-u" in sys.argv or "--use_factory_defaults" in sys.argv:
    logroot = get_temp_subdir()
else:
    #logroot = get_home_dir()
    logroot = get_dot_dir()

if is_frozen_exe:
    if logroot is None:
        logroot = os.path.splitdrive(sys.executable)[0]
        if logroot[-1] != os.sep:
            logroot += os.sep
    logname = os.path.split(sys.executable)[1]
else:
    logname = os.path.split(os.path.abspath(sys.argv[0]))[1]
logname = os.path.splitext(logname)[0] + '.log'
if logroot != '' and not os.path.exists(logroot):
    os.makedirs(logroot)
logpath = os.path.join(logroot, logname)


# becuase I'm generous.
STDERR = logging.CRITICAL + 10
logging.addLevelName(STDERR, 'STDERR')

# define a Handler which writes INFO messages or higher to the sys.stderr
console = logging.StreamHandler()
console.setLevel(logging.DEBUG)
# set a format which is simpler for console use
#formatter = logging.Formatter(u'%(name)-12s: %(levelname)-8s %(message)s')
formatter = logging.Formatter(u'%(message)s')
# tell the handler to use this format
console.setFormatter(formatter)
# add the handler to the root logger
logging.getLogger('').addHandler(console)

bt_log_fmt = logging.Formatter(u'[' + unicode(version) + u' %(asctime)s] %(levelname)-8s: %(message)s',
                               datefmt=u'%Y-%m-%d %H:%M:%S')

stderr_console = None
old_stderr = sys.stderr

def inject_main_logfile():
    # the main log file. log every kind of message, format properly,
    # rotate the log. someday - SocketHandler

    mainlog = logging.handlers.RotatingFileHandler(filename=logpath,
        mode='a', maxBytes=2**20, backupCount=1)
    mainlog.setFormatter(bt_log_fmt)
    mainlog.setLevel(logging.DEBUG)
    logger = logging.getLogger('')
    logging.getLogger('').addHandler(mainlog)
    logging.getLogger('').removeHandler(console)
    atexit_threads.register(lambda : logging.getLogger('').removeHandler(mainlog))

    global stderr_console
    if not is_frozen_exe:
        # write all stderr messages to stderr (unformatted)
        # as well as the main log (formatted)
        stderr_console = logging.StreamHandler(old_stderr)
        stderr_console.setLevel(STDERR)
        stderr_console.setFormatter(logging.Formatter(u'%(message)s'))
        logging.getLogger('').addHandler(stderr_console)

root_logger = logging.getLogger('')

class StderrProxy(StringIO):

    # whew. ugly. is there a simpler way to write this?
    # the goal is to stop every '\n' and flush to the log
    # otherwise keep buffering.
    def write(self, text, *args):
        lines = text.split('\n')
        for t in lines[:-1]:
            if len(t) > 0:
                StringIO.write(self, t)
            # the docs don't say it, but logging.log is new in 2.4
            #logging.log(STDERR, self.getvalue())
            root_logger.log(STDERR, self.getvalue())
            self.seek(0)
            self.truncate()
        if len(lines[-1]) > 0:
            StringIO.write(self, lines[-1])

sys.stderr = StderrProxy()
def reset_stderr():
    sys.stderr = old_stderr
atexit_threads.register(reset_stderr)






# By Greg Hazel and a smigden by Dave Harrison

debug = False
#debug = True

import os
import Queue
import socket
import itertools
import random
from pprint import pprint
from BitTorrent.platform import bttime
import BitTorrent.stackthreading as threading
from BitTorrent.HostIP import get_host_ip, get_host_ips


if os.name == 'nt':
    import win32icmp
    IP_TTL_EXPIRED_TRANSIT = 11013
    IP_SUCCESS = 0

def daemon_thread(target, args=()):
    t = threading.Thread(target=target, args=args)
    t.setDaemon(True)
    return t

def in_common(routes):
    """routes is a list of lists, each containing a route to a peer."""
    r = []
    branch = False
    for n in itertools.izip(*routes):

        # strip dead nodes
        f = [i for i in n if i != '*']

        # ignore all dead nodes
        if len(f) == 0:
            continue
        
        if len(set(f)) == 1:
            r.append(f[0])
        else:
            # more than one unique node, the tree has branched
            branch = True
            break
    return (branch, r)


class RTTMonitorBase(object):
    def __init__(self, new_rtt=None):
        self.instantanious_rtt = None
        self.min_rtt = None
        self.max_rtt = None
        def f(rtt):
            pass
        if new_rtt:
            self.new_rtt = new_rtt
        else:
            self.new_rtt = f

    def set_nodes_restart(self, nodes):
        pass

    def get_min_rtt(self):
        return self.min_rtt

    def get_max_rtt(self):
        return self.max_rtt

    def get_current_rtt(self):
        return self.instantanious_rtt


class RTTMonitorUnix(RTTMonitorBase):
    # I assume this will have a unix implementation some day
    pass

class Options(object):
    pass

def _set_min(x, y):
    if x is None:
        return y
    if y is None:
        return x
    return min(x, y)
_set_max = max

class RTTMonitorWin32(RTTMonitorBase):
    def __init__(self, new_rtt, interval = 0.5, timeout = 6.0):
        self.nodes = []
        self.timeout = int(1000 * timeout)
        self.interval = interval
        self.stop_event = threading.Event()
        self.abort_traceroute = threading.Event()
        self.finished_event = threading.Event()
        # the thread is finished because it hasn't started
        self.finished_event.set()

        RTTMonitorBase.__init__(self, new_rtt)

    def set_nodes_restart(self, nodes):
        self.nodes = []
        for node in nodes:
            self.add_node(node)
        t = threading.Thread(target=self.run, args=(list(self.nodes),))
        t.setDaemon(True)
        t.start()

    def add_node(self, node):
        self.nodes.append(node)

    def get_route(self, q, dst):
        try:
            dst = socket.gethostbyname(dst)
            self.traceroute(dst, self.timeout, lambda n : q.put((dst, n)))
        except socket.gaierror:
            # if hostbyname lookup fails, it's not a node we can use
            # maybe this should be a warning or something, but a downed
            # internet connection will cause a lot of these
            pass

    def run(self, nodes):
        
        q = Queue.Queue()

        dst = None
        # handy for hard-coding common node
        #dst = '68.87.195.50'; nodes = [dst,]; common = nodes
        if not dst: 
            threads = []
            for i in nodes:
                t = daemon_thread(target=self.get_route, args=(q, i, ))
                threads.append(t)
                t.start()

            waiter_done_event = threading.Event()
            def waiter(threads, waiter_done_event):
                try:
                    for thread in threads:
                        thread.join()
                except Exception, e:
                    print "waiter hiccupped", e
                waiter_done_event.set()
            waiting_thread = daemon_thread(target=waiter,
                                           args=(threads, waiter_done_event, ))
            waiting_thread.start()

            common = []
            routes = {}
            while not waiter_done_event.isSet():
                try:
                    msg = q.get(True, 1.0)
                except Queue.Empty:
                    pass
                else:
                    dst = msg[0]
                    # nodes appear in the queue in 
                    # increasing order of TTL.
                    new_node = msg[1]  
                    if dst not in routes:
                        l = []
                        routes[dst] = l
                    else:
                        l = routes[dst]
                    l.append(new_node)
                    branch, common = in_common(routes.values())
                    if branch:
                        break

            self.abort_traceroute.set()
            waiter_done_event.wait()
            self.abort_traceroute.clear()

            local_ips = get_host_ips()
            new_common = []
            for c in common:
                if c not in local_ips:
                    new_common.append(c)
            common = new_common
            
            if debug:
                pprint(common)

            if len(common) == 0:
                # this should be inspected, it's not a simple debug message
                if debug:
                    print "No common node", routes
                return

            del routes

            dst = common[-1]

        # kill the old thread
        self.stop_event.set()
        # wait for it to finish
        self.finished_event.wait()
        # clear to indicate we're running
        self.finished_event.clear()
        self.stop_event.clear()
        
        if debug:
            print "Farthest common hop [%s]" % dst

        # range can change if the node in common changes
        self.min_rtt = None
        self.max_rtt = None

        # Ping a representative peer but set the ttl to the length of the
        # common path so that the farthest common hop responds with
        # ICMP time exceeded.  (Some routers will send time exceeded 
        # messages, but they won't respond to ICMP pings directly)  
        representative = nodes[random.randrange(0, len(nodes))]
        if debug: 
            print "pinging representative %s ttl=%d" % (
                representative,len(common))
        try:            
            while not self.stop_event.isSet():
                start = bttime()
                rtt = self.ping(representative, 5000, ttl=len(common))
                self.instantanious_rtt = rtt

                self.min_rtt = _set_min(self.min_rtt, rtt)
                self.max_rtt = _set_max(self.max_rtt, rtt)

                delta = bttime() - start
                self.stop_event.wait(self.interval - delta)
                if debug: print "RTTMonitor.py: new_rtt %s" % rtt
                self.new_rtt(self.instantanious_rtt)
        except Exception, e:
            import traceback
            traceback.print_exc()
            print "ABORTING", e
            
        self.finished_event.set()

    def traceroute(self, dst, timeout, report=None):
        """If report is None then this returns the route as a list of IP
           addresses.  If report is not None then this calls report as each
           node is discovered in the path (e.g., if there are 6 hops in the
           path then report gets called 6 times)."""

        if debug:
            print "Tracing route to [%s]" % dst

        i = win32icmp.IcmpCreateFile()

        o = Options()

        route = None
        if report == None:
            route = []
            def add_node(node):
                route.append(node)
            report = add_node

        for ttl in xrange(64):
            o.Ttl = ttl
            try:
                if ttl == 0:
                    addr = get_host_ip()
                    status = -1
                    rtt = 0
                else:
                    addr, status, rtt = win32icmp.IcmpSendEcho(i, dst, None, o,
                                                               timeout)
                if debug:
                    print "ttl", ttl, "\t", rtt, "ms\t", addr
                report(addr)
                if status == IP_SUCCESS:
                    if debug:
                        print "Traceroute complete in", ttl, "hops"
                    break
            except Exception, e:
                report('*')
                if debug:
                    print "Hop", ttl, "failed:", unicode(e.args[0])
            if self.abort_traceroute.isSet():
                break

        win32icmp.IcmpCloseHandle(i)

        if route:
            return route

    def ping(self, dst, timeout, ttl = None):
        """Returns ICMP echo round-trip time to dst or returns None if a
           timeout occurs.  timeout is measured in milliseconds. 
          
           The TTL is useful if the caller wants to ping the router that 
           is a number of hops in the direction of the dst, e.g., when a 
           router will not respond to pings directly but will generate 
           ICMP Time Exceeded messages."""
        i = win32icmp.IcmpCreateFile()
        rtt = None

        try:
            o = None
            if ttl is not None:
                o = Options()
                o.Ttl = ttl

            addr, status, rtt = win32icmp.IcmpSendEcho(i, dst, None, o, timeout)
            if debug:
                if status == IP_SUCCESS:
                    print "Reply from", addr + ":", "time=" + str(rtt)
                elif status == IP_TTL_EXPIRED_TRANSIT:
                    print "Ping ttl expired %d: from %s time=%s" %(
                          status, str(addr), str(rtt))
                else:
                    print "Ping failed", status
        except Exception, e:
            if debug:
                print "Ping failed:", unicode(e.args[0])

        win32icmp.IcmpCloseHandle(i)

        return rtt

if os.name == 'nt':
    RTTMonitor = RTTMonitorWin32
else:
    RTTMonitor = RTTMonitorUnix
